/* valacallback.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "valacallback.h"
#include "valatypereference.h"
#include "valasourcereference.h"
#include "valatypeparameter.h"
#include "valaformalparameter.h"
#include "valamethod.h"
#include "valacodevisitor.h"
#include "valanamespace.h"

struct _ValaCallbackPrivate {
	ValaTypeReference* _return_type;
	gboolean _instance;
	GList* type_parameters;
	GList* parameters;
	char* cname;
};
#define VALA_CALLBACK_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CALLBACK, ValaCallbackPrivate))
enum  {
	VALA_CALLBACK_DUMMY_PROPERTY,
	VALA_CALLBACK_RETURN_TYPE,
	VALA_CALLBACK_INSTANCE,
};
static void vala_callback_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static char* vala_callback_real_get_cname (ValaDataType* base);
static gboolean vala_callback_real_is_reference_type (ValaDataType* base);
static void vala_callback_dispose (GObject * obj);

/**
 * Creates a new callback.
 *
 * @param name        callback type name
 * @param return_type return type
 * @param source      reference to source code
 * @return            newly created callback
 */
ValaCallback* vala_callback_new (char* _name, ValaTypeReference* _return_type, ValaSourceReference* source)
{
	g_return_val_if_fail (_return_type == NULL || VALA_IS_TYPE_REFERENCE (_return_type), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	GParameter * __params = g_new0 (GParameter, 3);
	GParameter * __params_it = __params;
	(__params_it->name = "name", g_value_init (&__params_it->value, G_TYPE_STRING), g_value_set_string (&__params_it->value, _name), __params_it++);
	(__params_it->name = "return-type", g_value_init (&__params_it->value, VALA_TYPE_TYPE_REFERENCE), g_value_set_object (&__params_it->value, _return_type), __params_it++);
	(__params_it->name = "source-reference", g_value_init (&__params_it->value, VALA_TYPE_SOURCE_REFERENCE), g_value_set_object (&__params_it->value, source), __params_it++);
	ValaCallback * self = g_object_newv (VALA_TYPE_CALLBACK, __params_it - __params, __params);
	return self;
}


/**
 * Appends the specified parameter to the list of type parameters.
 *
 * @param p a type parameter
 */
void vala_callback_add_type_parameter (ValaCallback* self, ValaTypeParameter* p)
{
	g_return_if_fail (VALA_IS_CALLBACK (self));
	g_return_if_fail (VALA_IS_TYPE_PARAMETER (p));
	self->priv->type_parameters = g_list_append (self->priv->type_parameters, g_object_ref (p));
	p->type = VALA_DATA_TYPE (self);
}


/**
 * Appends paramater to this callback function.
 *
 * @param param a formal parameter
 */
void vala_callback_add_parameter (ValaCallback* self, ValaFormalParameter* param)
{
	g_return_if_fail (VALA_IS_CALLBACK (self));
	g_return_if_fail (VALA_IS_FORMAL_PARAMETER (param));
	self->priv->parameters = g_list_append (self->priv->parameters, g_object_ref (param));
}


/**
 * Return copy of parameter list.
 *
 * @return parameter list
 */
GList* vala_callback_get_parameters (ValaCallback* self)
{
	g_return_val_if_fail (VALA_IS_CALLBACK (self), NULL);
	return g_list_copy (self->priv->parameters);
}


/**
 * Checks whether the arguments and return type of the specified method
 * matches this callback.
 *
 * @param m a method
 * @return  true if the specified method is compatible to this callback
 */
gboolean vala_callback_matches_method (ValaCallback* self, ValaMethod* m)
{
	g_return_val_if_fail (VALA_IS_CALLBACK (self), FALSE);
	g_return_val_if_fail (VALA_IS_METHOD (m), FALSE);
	if (!vala_type_reference_stricter (vala_method_get_return_type (m), vala_callback_get_return_type (self))) {
		return FALSE;
	}
	GList* method_params = vala_method_get_parameters (m);
	GList* method_params_it = method_params;
	{
		GList* param_it;
		for (param_it = self->priv->parameters; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				/* method is allowed to accept less arguments */
				if (method_params_it == NULL) {
					break;
				}
				ValaFormalParameter* method_param = VALA_FORMAL_PARAMETER (method_params_it->data);
				if (!vala_type_reference_stricter (vala_formal_parameter_get_type_reference (param), vala_formal_parameter_get_type_reference (method_param))) {
					gboolean __temp2;
					return (__temp2 = FALSE, (method_params == NULL ? NULL : (method_params = (g_list_free (method_params), NULL))), __temp2);
				}
				method_params_it = method_params_it->next;
			}
		}
	}
	/* method may not expect more arguments */
	if (method_params_it != NULL) {
		gboolean __temp3;
		return (__temp3 = FALSE, (method_params == NULL ? NULL : (method_params = (g_list_free (method_params), NULL))), __temp3);
	}
	gboolean __temp4;
	return (__temp4 = TRUE, (method_params == NULL ? NULL : (method_params = (g_list_free (method_params), NULL))), __temp4);
	(method_params == NULL ? NULL : (method_params = (g_list_free (method_params), NULL)));
}


static void vala_callback_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaCallback * self = VALA_CALLBACK (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_callback (visitor, self);
	{
		GList* p_it;
		for (p_it = self->priv->type_parameters; p_it != NULL; p_it = p_it->next) {
			ValaTypeParameter* p = p_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (p), visitor);
			}
		}
	}
	vala_code_node_accept (VALA_CODE_NODE (vala_callback_get_return_type (self)), visitor);
	{
		GList* param_it;
		for (param_it = self->priv->parameters; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (param), visitor);
			}
		}
	}
	vala_code_visitor_visit_end_callback (visitor, self);
}


static char* vala_callback_real_get_cname (ValaDataType* base)
{
	ValaCallback * self = VALA_CALLBACK (base);
	if (self->priv->cname == NULL) {
		char* __temp5 = NULL;
		self->priv->cname = (__temp5 = g_strdup_printf ("%s%s", vala_namespace_get_cprefix (VALA_DATA_TYPE (self)->namespace), vala_data_type_get_name (VALA_DATA_TYPE (self))), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp5);
	}
	return self->priv->cname;
}


static gboolean vala_callback_real_is_reference_type (ValaDataType* base)
{
	ValaCallback * self = VALA_CALLBACK (base);
	return FALSE;
}


ValaTypeReference* vala_callback_get_return_type (ValaCallback* self)
{
	g_return_val_if_fail (VALA_IS_CALLBACK (self), NULL);
	return self->priv->_return_type;
}


void vala_callback_set_return_type (ValaCallback* self, ValaTypeReference* value)
{
	g_return_if_fail (VALA_IS_CALLBACK (self));
	ValaTypeReference* __temp10 = NULL;
	ValaTypeReference* __temp9 = NULL;
	self->priv->_return_type = (__temp10 = (__temp9 = value, (__temp9 == NULL ? __temp9 : g_object_ref (__temp9))), (self->priv->_return_type == NULL ? NULL : (self->priv->_return_type = (g_object_unref (self->priv->_return_type), NULL))), __temp10);
}


gboolean vala_callback_get_instance (ValaCallback* self)
{
	g_return_val_if_fail (VALA_IS_CALLBACK (self), FALSE);
	return self->priv->_instance;
}


void vala_callback_set_instance (ValaCallback* self, gboolean value)
{
	g_return_if_fail (VALA_IS_CALLBACK (self));
	self->priv->_instance = value;
}


void vala_callback_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaCallback * self = VALA_CALLBACK (object);
	switch (property_id) {
		case VALA_CALLBACK_RETURN_TYPE:
		g_value_set_object (value, vala_callback_get_return_type (self));
		break;
		case VALA_CALLBACK_INSTANCE:
		g_value_set_boolean (value, vala_callback_get_instance (self));
		break;
	}
}


void vala_callback_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaCallback * self = VALA_CALLBACK (object);
	switch (property_id) {
		case VALA_CALLBACK_RETURN_TYPE:
		vala_callback_set_return_type (self, g_value_get_object (value));
		break;
		case VALA_CALLBACK_INSTANCE:
		vala_callback_set_instance (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_callback_class_init (ValaCallbackClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaCallbackPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_callback_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_callback_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_callback_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_callback_real_accept;
	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_callback_real_get_cname;
	VALA_DATA_TYPE_CLASS (klass)->is_reference_type = vala_callback_real_is_reference_type;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CALLBACK_RETURN_TYPE, g_param_spec_object ("return-type", "foo", "bar", VALA_TYPE_TYPE_REFERENCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CALLBACK_INSTANCE, g_param_spec_boolean ("instance", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_callback_init (ValaCallback * self)
{
	self->priv = VALA_CALLBACK_GET_PRIVATE (self);
}


static void vala_callback_dispose (GObject * obj)
{
	ValaCallback * self = VALA_CALLBACK (obj);
	(self->priv->_return_type == NULL ? NULL : (self->priv->_return_type = (g_object_unref (self->priv->_return_type), NULL)));
	(self->priv->type_parameters == NULL ? NULL : (self->priv->type_parameters = (g_list_foreach (self->priv->type_parameters, (GFunc) g_object_unref, NULL), g_list_free (self->priv->type_parameters), NULL)));
	(self->priv->parameters == NULL ? NULL : (self->priv->parameters = (g_list_foreach (self->priv->parameters, (GFunc) g_object_unref, NULL), g_list_free (self->priv->parameters), NULL)));
	(self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL)));
	ValaCallbackClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CALLBACK_CLASS (g_type_class_peek (VALA_TYPE_CALLBACK));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_callback_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCallbackClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_callback_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCallback), 0, (GInstanceInitFunc) vala_callback_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaCallback", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



