/* valaproperty.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "valaproperty.h"
#include "valatypereference.h"
#include "valapropertyaccessor.h"
#include "valaformalparameter.h"
#include "valasourcereference.h"
#include "valacodevisitor.h"
#include "valasymbol.h"
#include "valaclass.h"
#include "valanamespace.h"
#include "valaattribute.h"

struct _ValaPropertyPrivate {
	char* _name;
	ValaTypeReference* _type_reference;
	ValaPropertyAccessor* _get_accessor;
	ValaPropertyAccessor* _set_accessor;
	ValaMemberAccessibility _access;
	ValaFormalParameter* _this_parameter;
	gboolean _no_accessor_method;
};
#define VALA_PROPERTY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_PROPERTY, ValaPropertyPrivate))
enum  {
	VALA_PROPERTY_DUMMY_PROPERTY,
	VALA_PROPERTY_NAME,
	VALA_PROPERTY_TYPE_REFERENCE,
	VALA_PROPERTY_GET_ACCESSOR,
	VALA_PROPERTY_SET_ACCESSOR,
	VALA_PROPERTY_ACCESS,
	VALA_PROPERTY_THIS_PARAMETER,
	VALA_PROPERTY_NO_ACCESSOR_METHOD,
};
static void vala_property_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_property_dispose (GObject * obj);

/**
 * Creates a new property.
 *
 * @param name         property name
 * @param type         property type
 * @param get_accessor get accessor
 * @param set_accessor set/construct accessor
 * @param source       reference to source code
 * @return             newly created property
 */
ValaProperty* vala_property_new (char* _name, ValaTypeReference* type, ValaPropertyAccessor* _get_accessor, ValaPropertyAccessor* _set_accessor, ValaSourceReference* source)
{
	g_return_val_if_fail (_name != NULL, NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), NULL);
	g_return_val_if_fail (_get_accessor == NULL || VALA_IS_PROPERTY_ACCESSOR (_get_accessor), NULL);
	g_return_val_if_fail (_set_accessor == NULL || VALA_IS_PROPERTY_ACCESSOR (_set_accessor), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	GParameter * __params = g_new0 (GParameter, 5);
	GParameter * __params_it = __params;
	(__params_it->name = "name", g_value_init (&__params_it->value, G_TYPE_STRING), g_value_set_string (&__params_it->value, _name), __params_it++);
	(__params_it->name = "type-reference", g_value_init (&__params_it->value, VALA_TYPE_TYPE_REFERENCE), g_value_set_object (&__params_it->value, type), __params_it++);
	(__params_it->name = "get-accessor", g_value_init (&__params_it->value, VALA_TYPE_PROPERTY_ACCESSOR), g_value_set_object (&__params_it->value, _get_accessor), __params_it++);
	(__params_it->name = "set-accessor", g_value_init (&__params_it->value, VALA_TYPE_PROPERTY_ACCESSOR), g_value_set_object (&__params_it->value, _set_accessor), __params_it++);
	(__params_it->name = "source-reference", g_value_init (&__params_it->value, VALA_TYPE_SOURCE_REFERENCE), g_value_set_object (&__params_it->value, source), __params_it++);
	ValaProperty * self = g_object_newv (VALA_TYPE_PROPERTY, __params_it - __params, __params);
	return self;
}


static void vala_property_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaProperty * self = VALA_PROPERTY (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_property (visitor, self);
	vala_code_node_accept (VALA_CODE_NODE (vala_property_get_type_reference (self)), visitor);
	if (vala_property_get_get_accessor (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_property_get_get_accessor (self)), visitor);
	}
	if (vala_property_get_set_accessor (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_property_get_set_accessor (self)), visitor);
	}
	vala_code_visitor_visit_end_property (visitor, self);
}


/**
 * Returns the C name of this property in upper case. Words are
 * separated by underscores. The upper case C name of the class is
 * prefix of the result.
 *
 * @return the upper case name to be used in C code
 */
char* vala_property_get_upper_case_cname (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	char* __temp2 = NULL;
	char* __temp1 = NULL;
	char* __temp0 = NULL;
	char* __temp3 = NULL;
	return (__temp3 = g_utf8_strup ((__temp2 = g_strdup_printf ("%s_%s", (__temp0 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE ((VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (self))))))), NULL)), (__temp1 = vala_namespace_camel_case_to_lower_case (vala_property_get_name (self))))), -1), (__temp2 == NULL ? NULL : (__temp2 = (g_free (__temp2), NULL))), (__temp1 == NULL ? NULL : (__temp1 = (g_free (__temp1), NULL))), (__temp0 == NULL ? NULL : (__temp0 = (g_free (__temp0), NULL))), __temp3);
}


/**
 * Returns the string literal of this property to be used in C code.
 *
 * @return string literal to be used in C code
 */
ValaCCodeConstant* vala_property_get_canonical_cconstant (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	GString* str = g_string_new ("\"");
	char* i = vala_property_get_name (self);
	while (g_utf8_strlen (i, -1) > 0) {
		gunichar c = g_utf8_get_char (i);
		if (c == '_') {
			g_string_append_c (str, '-');
		} else {
			g_string_append_unichar (str, c);
		}
		i = g_utf8_next_char (i);
	}
	g_string_append_c (str, '"');
	ValaCCodeConstant* __temp5 = NULL;
	return (__temp5 = vala_ccode_constant_new (str->str), (str == NULL ? NULL : (str = (g_string_free (str, TRUE), NULL))), __temp5);
	(str == NULL ? NULL : (str = (g_string_free (str, TRUE), NULL)));
}


/**
 * Process all associated attributes.
 */
void vala_property_process_attributes (ValaProperty* self)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	{
		GList* a_it;
		for (a_it = VALA_CODE_NODE (self)->attributes; a_it != NULL; a_it = a_it->next) {
			ValaAttribute* a = a_it->data;
			{
				if (g_utf8_collate (vala_attribute_get_name (a), "NoAccessorMethod") == 0) {
					vala_property_set_no_accessor_method (self, TRUE);
				}
			}
		}
	}
}


char* vala_property_get_name (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	return self->priv->_name;
}


void vala_property_set_name (ValaProperty* self, char* value)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	char* __temp7 = NULL;
	self->priv->_name = (__temp7 = g_strdup (value), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp7);
}


ValaTypeReference* vala_property_get_type_reference (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	return self->priv->_type_reference;
}


void vala_property_set_type_reference (ValaProperty* self, ValaTypeReference* value)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	ValaTypeReference* __temp9 = NULL;
	self->priv->_type_reference = (__temp9 = g_object_ref (value), (self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL))), __temp9);
}


ValaPropertyAccessor* vala_property_get_get_accessor (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	return self->priv->_get_accessor;
}


void vala_property_set_get_accessor (ValaProperty* self, ValaPropertyAccessor* value)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	ValaPropertyAccessor* __temp12 = NULL;
	ValaPropertyAccessor* __temp11 = NULL;
	self->priv->_get_accessor = (__temp12 = (__temp11 = value, (__temp11 == NULL ? __temp11 : g_object_ref (__temp11))), (self->priv->_get_accessor == NULL ? NULL : (self->priv->_get_accessor = (g_object_unref (self->priv->_get_accessor), NULL))), __temp12);
}


ValaPropertyAccessor* vala_property_get_set_accessor (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	return self->priv->_set_accessor;
}


void vala_property_set_set_accessor (ValaProperty* self, ValaPropertyAccessor* value)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	ValaPropertyAccessor* __temp15 = NULL;
	ValaPropertyAccessor* __temp14 = NULL;
	self->priv->_set_accessor = (__temp15 = (__temp14 = value, (__temp14 == NULL ? __temp14 : g_object_ref (__temp14))), (self->priv->_set_accessor == NULL ? NULL : (self->priv->_set_accessor = (g_object_unref (self->priv->_set_accessor), NULL))), __temp15);
}


ValaMemberAccessibility vala_property_get_access (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), 0);
	return self->priv->_access;
}


void vala_property_set_access (ValaProperty* self, ValaMemberAccessibility value)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	self->priv->_access = value;
}


ValaFormalParameter* vala_property_get_this_parameter (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	return self->priv->_this_parameter;
}


void vala_property_set_this_parameter (ValaProperty* self, ValaFormalParameter* value)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	ValaFormalParameter* __temp19 = NULL;
	ValaFormalParameter* __temp18 = NULL;
	self->priv->_this_parameter = (__temp19 = (__temp18 = value, (__temp18 == NULL ? __temp18 : g_object_ref (__temp18))), (self->priv->_this_parameter == NULL ? NULL : (self->priv->_this_parameter = (g_object_unref (self->priv->_this_parameter), NULL))), __temp19);
}


gboolean vala_property_get_no_accessor_method (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), FALSE);
	return self->priv->_no_accessor_method;
}


void vala_property_set_no_accessor_method (ValaProperty* self, gboolean value)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	self->priv->_no_accessor_method = value;
}


void vala_property_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaProperty * self = VALA_PROPERTY (object);
	switch (property_id) {
		case VALA_PROPERTY_NAME:
		g_value_set_string (value, vala_property_get_name (self));
		break;
		case VALA_PROPERTY_TYPE_REFERENCE:
		g_value_set_object (value, vala_property_get_type_reference (self));
		break;
		case VALA_PROPERTY_GET_ACCESSOR:
		g_value_set_object (value, vala_property_get_get_accessor (self));
		break;
		case VALA_PROPERTY_SET_ACCESSOR:
		g_value_set_object (value, vala_property_get_set_accessor (self));
		break;
		case VALA_PROPERTY_ACCESS:
		g_value_set_int (value, vala_property_get_access (self));
		break;
		case VALA_PROPERTY_THIS_PARAMETER:
		g_value_set_object (value, vala_property_get_this_parameter (self));
		break;
		case VALA_PROPERTY_NO_ACCESSOR_METHOD:
		g_value_set_boolean (value, vala_property_get_no_accessor_method (self));
		break;
	}
}


void vala_property_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaProperty * self = VALA_PROPERTY (object);
	switch (property_id) {
		case VALA_PROPERTY_NAME:
		vala_property_set_name (self, g_value_get_string (value));
		break;
		case VALA_PROPERTY_TYPE_REFERENCE:
		vala_property_set_type_reference (self, g_value_get_object (value));
		break;
		case VALA_PROPERTY_GET_ACCESSOR:
		vala_property_set_get_accessor (self, g_value_get_object (value));
		break;
		case VALA_PROPERTY_SET_ACCESSOR:
		vala_property_set_set_accessor (self, g_value_get_object (value));
		break;
		case VALA_PROPERTY_ACCESS:
		vala_property_set_access (self, g_value_get_int (value));
		break;
		case VALA_PROPERTY_THIS_PARAMETER:
		vala_property_set_this_parameter (self, g_value_get_object (value));
		break;
		case VALA_PROPERTY_NO_ACCESSOR_METHOD:
		vala_property_set_no_accessor_method (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_property_class_init (ValaPropertyClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaPropertyPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_property_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_property_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_property_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_property_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_TYPE_REFERENCE, g_param_spec_object ("type-reference", "foo", "bar", VALA_TYPE_TYPE_REFERENCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_GET_ACCESSOR, g_param_spec_object ("get-accessor", "foo", "bar", VALA_TYPE_PROPERTY_ACCESSOR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_SET_ACCESSOR, g_param_spec_object ("set-accessor", "foo", "bar", VALA_TYPE_PROPERTY_ACCESSOR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_ACCESS, g_param_spec_int ("access", "foo", "bar", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_THIS_PARAMETER, g_param_spec_object ("this-parameter", "foo", "bar", VALA_TYPE_FORMAL_PARAMETER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_NO_ACCESSOR_METHOD, g_param_spec_boolean ("no-accessor-method", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_property_init (ValaProperty * self)
{
	self->priv = VALA_PROPERTY_GET_PRIVATE (self);
}


static void vala_property_dispose (GObject * obj)
{
	ValaProperty * self = VALA_PROPERTY (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	(self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL)));
	(self->priv->_get_accessor == NULL ? NULL : (self->priv->_get_accessor = (g_object_unref (self->priv->_get_accessor), NULL)));
	(self->priv->_set_accessor == NULL ? NULL : (self->priv->_set_accessor = (g_object_unref (self->priv->_set_accessor), NULL)));
	(self->priv->_this_parameter == NULL ? NULL : (self->priv->_this_parameter = (g_object_unref (self->priv->_this_parameter), NULL)));
	ValaPropertyClass * klass;
	GObjectClass * parent_class;
	klass = VALA_PROPERTY_CLASS (g_type_class_peek (VALA_TYPE_PROPERTY));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_property_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaPropertyClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_property_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaProperty), 0, (GInstanceInitFunc) vala_property_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaProperty", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



