/* valasemanticanalyzer.vala
 *
 * Copyright (C) 2006  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <rasa@gmx.ch>
 */

#include "valasemanticanalyzer.h"
#include "valamemberaccessibility.h"
#include "valastatement.h"
#include "valareport.h"
#include "valasourcereference.h"
#include "valainterface.h"
#include "valacallback.h"
#include "valaliteral.h"
#include "valaconstant.h"
#include "valaenumvalue.h"
#include "valanamespacereference.h"
#include "valaarray.h"
#include "valasignal.h"

struct _ValaSemanticAnalyzerPrivate {
	gboolean _memory_management;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaSourceFile* current_source_file;
	ValaTypeReference* current_return_type;
	GList* current_using_directives;
	ValaTypeReference* bool_type;
	ValaTypeReference* string_type;
	ValaTypeReference* int_type;
	ValaDataType* pointer_type;
	ValaDataType* initially_unowned_type;
	int next_lambda_id;
};
#define VALA_SEMANTIC_ANALYZER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzerPrivate))
enum  {
	VALA_SEMANTIC_ANALYZER_DUMMY_PROPERTY,
	VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT,
};
static void vala_semantic_analyzer_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vala_semantic_analyzer_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vala_semantic_analyzer_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_semantic_analyzer_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_semantic_analyzer_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_semantic_analyzer_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_semantic_analyzer_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_semantic_analyzer_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_semantic_analyzer_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vala_semantic_analyzer_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_semantic_analyzer_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_semantic_analyzer_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p);
static void vala_semantic_analyzer_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_semantic_analyzer_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_semantic_analyzer_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_semantic_analyzer_real_visit_begin_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_semantic_analyzer_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_semantic_analyzer_real_visit_begin_destructor (ValaCodeVisitor* base, ValaDestructor* d);
static void vala_semantic_analyzer_real_visit_end_destructor (ValaCodeVisitor* base, ValaDestructor* d);
static void vala_semantic_analyzer_real_visit_named_argument (ValaCodeVisitor* base, ValaNamedArgument* n);
static void vala_semantic_analyzer_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_semantic_analyzer_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_semantic_analyzer_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl);
static void vala_semantic_analyzer_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt);
static void vala_semantic_analyzer_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt);
static void vala_semantic_analyzer_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt);
static void vala_semantic_analyzer_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt);
static void vala_semantic_analyzer_real_visit_begin_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt);
static void vala_semantic_analyzer_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt);
static void vala_semantic_analyzer_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr);
static void vala_semantic_analyzer_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr);
static void vala_semantic_analyzer_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr);
static void vala_semantic_analyzer_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr);
static void vala_semantic_analyzer_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr);
static void vala_semantic_analyzer_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr);
static void vala_semantic_analyzer_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr);
static ValaTypeReference* vala_semantic_analyzer_get_static_type_for_node (ValaSemanticAnalyzer* self, ValaCodeNode* node);
static ValaSymbol* vala_semantic_analyzer_symbol_lookup_inherited (ValaSemanticAnalyzer* self, ValaSymbol* sym, char* name);
static void vala_semantic_analyzer_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr);
static ValaDataType* vala_semantic_analyzer_find_parent_type (ValaSemanticAnalyzer* self, ValaSymbol* sym);
static void vala_semantic_analyzer_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr);
static gboolean vala_semantic_analyzer_is_type_compatible (ValaSemanticAnalyzer* self, ValaTypeReference* expression_type, ValaTypeReference* expected_type);
static void vala_semantic_analyzer_real_visit_begin_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr);
static gboolean vala_semantic_analyzer_check_arguments (ValaSemanticAnalyzer* self, ValaExpression* expr, ValaSymbol* msym, GList* params, GList* args);
static void vala_semantic_analyzer_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr);
static void vala_semantic_analyzer_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr);
static void vala_semantic_analyzer_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr);
static void vala_semantic_analyzer_real_visit_end_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr);
static void vala_semantic_analyzer_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr);
static void vala_semantic_analyzer_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr);
static gboolean vala_semantic_analyzer_check_binary_type (ValaSemanticAnalyzer* self, ValaBinaryExpression* expr, char* operation);
static void vala_semantic_analyzer_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr);
static void vala_semantic_analyzer_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr);
static void vala_semantic_analyzer_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr);
static char* vala_semantic_analyzer_get_lambda_name (ValaSemanticAnalyzer* self);
static ValaMethod* vala_semantic_analyzer_find_current_method (ValaSemanticAnalyzer* self);
static void vala_semantic_analyzer_real_visit_begin_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l);
static void vala_semantic_analyzer_real_visit_begin_assignment (ValaCodeVisitor* base, ValaAssignment* a);
static void vala_semantic_analyzer_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a);
static void vala_semantic_analyzer_dispose (GObject * obj);

ValaSemanticAnalyzer* vala_semantic_analyzer_new (gboolean manage_memory)
{
	GParameter * __params = g_new0 (GParameter, 1);
	GParameter * __params_it = __params;
	(__params_it->name = "memory-management", g_value_init (&__params_it->value, G_TYPE_BOOLEAN), g_value_set_boolean (&__params_it->value, manage_memory), __params_it++);
	ValaSemanticAnalyzer * self = g_object_newv (VALA_TYPE_SEMANTIC_ANALYZER, __params_it - __params, __params);
	return self;
}


/**
 * Analyze and check code in the specified context.
 *
 * @param context a code context
 */
void vala_semantic_analyzer_analyze (ValaSemanticAnalyzer* self, ValaCodeContext* context)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	ValaSymbol* __temp0 = NULL;
	self->priv->root_symbol = (__temp0 = g_object_ref (vala_code_context_get_root (context)), (self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL))), __temp0);
	ValaTypeReference* __temp1 = NULL;
	self->priv->bool_type = (__temp1 = vala_type_reference_new (), (self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL))), __temp1);
	vala_type_reference_set_data_type (self->priv->bool_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "bool"))));
	ValaTypeReference* __temp2 = NULL;
	self->priv->string_type = (__temp2 = vala_type_reference_new (), (self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL))), __temp2);
	vala_type_reference_set_data_type (self->priv->string_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "string"))));
	ValaDataType* __temp4 = NULL;
	ValaDataType* __temp3 = NULL;
	self->priv->pointer_type = (__temp4 = (__temp3 = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "pointer"))), (__temp3 == NULL ? __temp3 : g_object_ref (__temp3))), (self->priv->pointer_type == NULL ? NULL : (self->priv->pointer_type = (g_object_unref (self->priv->pointer_type), NULL))), __temp4);
	ValaTypeReference* __temp5 = NULL;
	self->priv->int_type = (__temp5 = vala_type_reference_new (), (self->priv->int_type == NULL ? NULL : (self->priv->int_type = (g_object_unref (self->priv->int_type), NULL))), __temp5);
	vala_type_reference_set_data_type (self->priv->int_type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int"))));
	ValaSymbol* glib_ns = vala_symbol_lookup (self->priv->root_symbol, "GLib");
	ValaDataType* __temp7 = NULL;
	ValaDataType* __temp6 = NULL;
	self->priv->initially_unowned_type = (__temp7 = (__temp6 = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (glib_ns, "InitiallyUnowned"))), (__temp6 == NULL ? __temp6 : g_object_ref (__temp6))), (self->priv->initially_unowned_type == NULL ? NULL : (self->priv->initially_unowned_type = (g_object_unref (self->priv->initially_unowned_type), NULL))), __temp7);
	ValaSymbol* __temp9 = NULL;
	ValaSymbol* __temp8 = NULL;
	self->priv->current_symbol = (__temp9 = (__temp8 = self->priv->root_symbol, (__temp8 == NULL ? __temp8 : g_object_ref (__temp8))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp9);
	vala_code_context_accept (context, VALA_CODE_VISITOR (self));
}


static void vala_semantic_analyzer_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* file)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (file));
	ValaSourceFile* __temp10 = NULL;
	self->priv->current_source_file = (__temp10 = g_object_ref (file), (self->priv->current_source_file == NULL ? NULL : (self->priv->current_source_file = (g_object_unref (self->priv->current_source_file), NULL))), __temp10);
	GList* __temp11 = NULL;
	self->priv->current_using_directives = (__temp11 = vala_source_file_get_using_directives (file), (self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL))), __temp11);
	self->priv->next_lambda_id = 0;
}


static void vala_semantic_analyzer_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* file)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (file));
	GList* __temp12 = NULL;
	self->priv->current_using_directives = (__temp12 = NULL, (self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL))), __temp12);
}


static void vala_semantic_analyzer_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	ValaSymbol* __temp14 = NULL;
	ValaSymbol* __temp13 = NULL;
	self->priv->current_symbol = (__temp14 = (__temp13 = vala_code_node_get_symbol (VALA_CODE_NODE (ns)), (__temp13 == NULL ? __temp13 : g_object_ref (__temp13))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp14);
}


static void vala_semantic_analyzer_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	ValaSymbol* __temp16 = NULL;
	ValaSymbol* __temp15 = NULL;
	self->priv->current_symbol = (__temp16 = (__temp15 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp15 == NULL ? __temp15 : g_object_ref (__temp15))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp16);
}


static void vala_semantic_analyzer_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* __temp18 = NULL;
	ValaSymbol* __temp17 = NULL;
	self->priv->current_symbol = (__temp18 = (__temp17 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp17 == NULL ? __temp17 : g_object_ref (__temp17))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp18);
	if (vala_class_get_base_class (cl) != NULL) {
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_class_get_base_class (cl))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL);
	}
}


static void vala_semantic_analyzer_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* __temp20 = NULL;
	ValaSymbol* __temp19 = NULL;
	self->priv->current_symbol = (__temp20 = (__temp19 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp19 == NULL ? __temp19 : g_object_ref (__temp19))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp20);
}


static void vala_semantic_analyzer_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaSymbol* __temp22 = NULL;
	ValaSymbol* __temp21 = NULL;
	self->priv->current_symbol = (__temp22 = (__temp21 = vala_code_node_get_symbol (VALA_CODE_NODE (st)), (__temp21 == NULL ? __temp21 : g_object_ref (__temp21))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp22);
}


static void vala_semantic_analyzer_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaSymbol* __temp24 = NULL;
	ValaSymbol* __temp23 = NULL;
	self->priv->current_symbol = (__temp24 = (__temp23 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp23 == NULL ? __temp23 : g_object_ref (__temp23))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp24);
}


static void vala_semantic_analyzer_real_visit_field (ValaCodeVisitor* base, ValaField* f)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FIELD (f));
	if (f->access == VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		if (vala_type_reference_get_data_type (vala_field_get_type_reference (f)) != NULL) {
			/* is null if it references a type parameter */
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_field_get_type_reference (f)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
		}
	} else {
		if (vala_type_reference_get_data_type (vala_field_get_type_reference (f)) != NULL) {
			/* is null if it references a type parameter */
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_field_get_type_reference (f)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
		}
	}
}


static void vala_semantic_analyzer_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	if (vala_method_get_construction (m)) {
		ValaTypeReference* __temp25 = NULL;
		vala_method_set_return_type (m, (__temp25 = vala_type_reference_new ()));
		(__temp25 == NULL ? NULL : (__temp25 = (g_object_unref (__temp25), NULL)));
		vala_type_reference_set_data_type (vala_method_get_return_type (m), VALA_DATA_TYPE (vala_symbol_get_node (self->priv->current_symbol)));
		vala_type_reference_set_transfers_ownership (vala_method_get_return_type (m), TRUE);
		if (VALA_IS_CLASS (vala_symbol_get_node (self->priv->current_symbol))) {
			/* check for floating reference*/
			ValaClass* cl = VALA_CLASS (vala_symbol_get_node (self->priv->current_symbol));
			while (cl != NULL) {
				if (cl == self->priv->initially_unowned_type) {
					vala_type_reference_set_floating_reference (vala_method_get_return_type (m), TRUE);
					break;
				}
				cl = vala_class_get_base_class (cl);
			}
		}
		if (vala_method_get_body (m) != NULL) {
			vala_statement_set_construction (VALA_STATEMENT (vala_method_get_body (m)), TRUE);
		}
	}
	ValaSymbol* __temp27 = NULL;
	ValaSymbol* __temp26 = NULL;
	self->priv->current_symbol = (__temp27 = (__temp26 = vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp26 == NULL ? __temp26 : g_object_ref (__temp26))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp27);
	ValaTypeReference* __temp29 = NULL;
	ValaTypeReference* __temp28 = NULL;
	self->priv->current_return_type = (__temp29 = (__temp28 = vala_method_get_return_type (m), (__temp28 == NULL ? __temp28 : g_object_ref (__temp28))), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp29);
	if (vala_type_reference_get_data_type (vala_method_get_return_type (m)) != NULL) {
		/* is null if it is void or a reference to a type parameter */
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_method_get_return_type (m)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
	}
}


static void vala_semantic_analyzer_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp31 = NULL;
	ValaSymbol* __temp30 = NULL;
	self->priv->current_symbol = (__temp31 = (__temp30 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp30 == NULL ? __temp30 : g_object_ref (__temp30))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp31);
	ValaTypeReference* __temp32 = NULL;
	self->priv->current_return_type = (__temp32 = NULL, (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp32);
	if (VALA_IS_METHOD (vala_symbol_get_node (vala_symbol_get_parent_symbol (self->priv->current_symbol)))) {
		/* lambda expressions produce nested methods */
		ValaMethod* up_method = VALA_METHOD (vala_symbol_get_node (vala_symbol_get_parent_symbol (self->priv->current_symbol)));
		ValaTypeReference* __temp34 = NULL;
		ValaTypeReference* __temp33 = NULL;
		self->priv->current_return_type = (__temp34 = (__temp33 = vala_method_get_return_type (up_method), (__temp33 == NULL ? __temp33 : g_object_ref (__temp33))), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp34);
	}
	if (vala_method_get_is_virtual (m) || vala_method_get_overrides (m)) {
		if (VALA_IS_CLASS (vala_symbol_get_node (self->priv->current_symbol))) {
			ValaClass* cl = VALA_CLASS (vala_symbol_get_node (self->priv->current_symbol));
			ValaClass* base_class = NULL;
			for (base_class = cl; base_class != NULL; base_class = vala_class_get_base_class (base_class)) {
				ValaSymbol* sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (base_class)), vala_method_get_name (m));
				if (sym != NULL && VALA_IS_METHOD (vala_symbol_get_node (sym))) {
					ValaMethod* base_method = VALA_METHOD (vala_symbol_get_node (sym));
					if (vala_method_get_is_abstract (base_method) || vala_method_get_is_virtual (base_method)) {
						if (!vala_method_equals (m, base_method)) {
							vala_code_node_set_error (VALA_CODE_NODE (m), TRUE);
							char* __temp37 = NULL;
							char* __temp36 = NULL;
							char* __temp35 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp37 = g_strdup_printf ("Return type and or parameters of overridding method `%s' do not match overridden method `%s'.", (__temp35 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp36 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (base_method)))))));
							(__temp37 == NULL ? NULL : (__temp37 = (g_free (__temp37), NULL)));
							(__temp36 == NULL ? NULL : (__temp36 = (g_free (__temp36), NULL)));
							(__temp35 == NULL ? NULL : (__temp35 = (g_free (__temp35), NULL)));
							return;
						}
						vala_method_set_base_method (m, base_method);
						break;
					}
				}
			}
			if (vala_method_get_base_method (m) == NULL) {
				{
					GList* type_it;
					for (type_it = vala_class_get_base_types (cl); type_it != NULL; type_it = type_it->next) {
						ValaTypeReference* type = type_it->data;
						{
							/* FIXME: also look at interfaces implemented
							 * by one of the base types
							 */
							if (VALA_IS_INTERFACE (vala_type_reference_get_data_type (type))) {
								ValaInterface* iface = VALA_INTERFACE (vala_type_reference_get_data_type (type));
								ValaSymbol* sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (iface)), vala_method_get_name (m));
								if (sym != NULL && VALA_IS_METHOD (vala_symbol_get_node (sym))) {
									ValaMethod* base_method = VALA_METHOD (vala_symbol_get_node (sym));
									if (vala_method_get_is_abstract (base_method) || vala_method_get_is_virtual (base_method)) {
										vala_method_set_base_method (m, base_method);
										break;
									}
								}
							}
						}
					}
				}
			}
			if (vala_method_get_base_method (m) == NULL) {
				char* __temp39 = NULL;
				char* __temp38 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp39 = g_strdup_printf ("%s: no suitable method found to override", (__temp38 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))));
				(__temp39 == NULL ? NULL : (__temp39 = (g_free (__temp39), NULL)));
				(__temp38 == NULL ? NULL : (__temp38 = (g_free (__temp38), NULL)));
			}
		} else {
			if (VALA_IS_STRUCT (vala_symbol_get_node (self->priv->current_symbol))) {
				char* __temp41 = NULL;
				char* __temp40 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp41 = g_strdup_printf ("A struct member `%s' cannot be marked as override, virtual, or abstract", (__temp40 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))));
				(__temp41 == NULL ? NULL : (__temp41 = (g_free (__temp41), NULL)));
				(__temp40 == NULL ? NULL : (__temp40 = (g_free (__temp40), NULL)));
				return;
			}
		}
	}
	if (vala_method_get_construction (m) && vala_method_get_body (m) != NULL) {
		int n_params = 0;
		{
			GList* stmt_it;
			for (stmt_it = vala_block_get_statements (vala_method_get_body (m)); stmt_it != NULL; stmt_it = stmt_it->next) {
				ValaStatement* stmt = stmt_it->data;
				{
					int params = vala_statement_get_number_of_set_construction_parameters (stmt);
					if (params == -1) {
						break;
					}
					n_params += params;
					vala_statement_set_construction (stmt, TRUE);
				}
			}
		}
		vala_method_set_n_construction_params (m, n_params);
	}
}


static void vala_semantic_analyzer_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FORMAL_PARAMETER (p));
	if (!vala_formal_parameter_get_ellipsis (p)) {
		if (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)) != NULL) {
			/* is null if it references a type parameter */
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
		}
	}
}


static void vala_semantic_analyzer_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) != NULL) {
		/* is null if it references a type parameter */
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	}
}


static void vala_semantic_analyzer_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (acc)))));
	if (vala_property_accessor_get_readable (acc)) {
		ValaTypeReference* __temp42 = NULL;
		self->priv->current_return_type = (__temp42 = g_object_ref (vala_property_get_type_reference (prop)), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp42);
	}
}


static void vala_semantic_analyzer_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaTypeReference* __temp43 = NULL;
	self->priv->current_return_type = (__temp43 = NULL, (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp43);
}


static void vala_semantic_analyzer_real_visit_begin_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaSymbol* __temp45 = NULL;
	ValaSymbol* __temp44 = NULL;
	self->priv->current_symbol = (__temp45 = (__temp44 = vala_code_node_get_symbol (VALA_CODE_NODE (c)), (__temp44 == NULL ? __temp44 : g_object_ref (__temp44))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp45);
}


static void vala_semantic_analyzer_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaSymbol* __temp47 = NULL;
	ValaSymbol* __temp46 = NULL;
	self->priv->current_symbol = (__temp47 = (__temp46 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp46 == NULL ? __temp46 : g_object_ref (__temp46))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp47);
}


static void vala_semantic_analyzer_real_visit_begin_destructor (ValaCodeVisitor* base, ValaDestructor* d)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_DESTRUCTOR (d));
	ValaSymbol* __temp49 = NULL;
	ValaSymbol* __temp48 = NULL;
	self->priv->current_symbol = (__temp49 = (__temp48 = vala_code_node_get_symbol (VALA_CODE_NODE (d)), (__temp48 == NULL ? __temp48 : g_object_ref (__temp48))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp49);
}


static void vala_semantic_analyzer_real_visit_end_destructor (ValaCodeVisitor* base, ValaDestructor* d)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_DESTRUCTOR (d));
	ValaSymbol* __temp51 = NULL;
	ValaSymbol* __temp50 = NULL;
	self->priv->current_symbol = (__temp51 = (__temp50 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp50 == NULL ? __temp50 : g_object_ref (__temp50))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp51);
}


static void vala_semantic_analyzer_real_visit_named_argument (ValaCodeVisitor* base, ValaNamedArgument* n)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NAMED_ARGUMENT (n));
}


static void vala_semantic_analyzer_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	ValaSymbol* __temp53 = NULL;
	ValaSymbol* __temp52 = NULL;
	self->priv->current_symbol = (__temp53 = (__temp52 = vala_code_node_get_symbol (VALA_CODE_NODE (b)), (__temp52 == NULL ? __temp52 : g_object_ref (__temp52))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp53);
}


static void vala_semantic_analyzer_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	{
		GList* decl_it;
		for (decl_it = vala_block_get_local_variables (b); decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				vala_symbol_set_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl)), FALSE);
			}
		}
	}
	ValaSymbol* __temp55 = NULL;
	ValaSymbol* __temp54 = NULL;
	self->priv->current_symbol = (__temp55 = (__temp54 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp54 == NULL ? __temp54 : g_object_ref (__temp54))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp55);
}


static void vala_semantic_analyzer_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_VARIABLE_DECLARATOR (decl));
	if (vala_variable_declarator_get_type_reference (decl) == NULL) {
		/* var type */
		if (vala_variable_declarator_get_initializer (decl) == NULL) {
			vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "var declaration not allowed without initializer");
			return;
		}
		if (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)) == NULL) {
			vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "var declaration not allowed with non-typed initializer");
			return;
		}
		ValaTypeReference* __temp56 = NULL;
		vala_variable_declarator_set_type_reference (decl, (__temp56 = vala_type_reference_copy (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)))));
		(__temp56 == NULL ? NULL : (__temp56 = (g_object_unref (__temp56), NULL)));
		vala_type_reference_set_takes_ownership (vala_variable_declarator_get_type_reference (decl), vala_type_reference_get_transfers_ownership (vala_variable_declarator_get_type_reference (decl)));
		vala_type_reference_set_transfers_ownership (vala_variable_declarator_get_type_reference (decl), FALSE);
	}
	if (vala_variable_declarator_get_initializer (decl) != NULL) {
		if (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)) == NULL) {
			if (!(VALA_IS_MEMBER_ACCESS (vala_variable_declarator_get_initializer (decl)))) {
				vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE);
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "expression type not allowed as initializer");
				return;
			}
			if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_variable_declarator_get_initializer (decl)))) && VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
				ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_variable_declarator_get_initializer (decl))));
				ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)));
				/* check whether method matches callback type */
				if (!vala_callback_matches_method (cb, m)) {
					vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE);
					char* __temp59 = NULL;
					char* __temp58 = NULL;
					char* __temp57 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), (__temp59 = g_strdup_printf ("declaration of method `%s' doesn't match declaration of callback `%s'", (__temp57 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp58 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cb)))))));
					(__temp59 == NULL ? NULL : (__temp59 = (g_free (__temp59), NULL)));
					(__temp58 == NULL ? NULL : (__temp58 = (g_free (__temp58), NULL)));
					(__temp57 == NULL ? NULL : (__temp57 = (g_free (__temp57), NULL)));
					return;
				}
				vala_expression_set_static_type (vala_variable_declarator_get_initializer (decl), vala_variable_declarator_get_type_reference (decl));
			} else {
				vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE);
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "expression type not allowed as initializer");
				return;
			}
		}
		if (vala_semantic_analyzer_get_memory_management (self)) {
			if (vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)))) {
				/* rhs transfers ownership of the expression */
				if (!vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl))) {
					/* lhs doesn't own the value
					 * promote lhs type */
					vala_type_reference_set_takes_ownership (vala_variable_declarator_get_type_reference (decl), TRUE);
				}
			}
		}
	}
	if (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)) != NULL) {
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	}
	ValaSymbol* __temp60 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (decl), (__temp60 = vala_symbol_new (VALA_CODE_NODE (decl))));
	(__temp60 == NULL ? NULL : (__temp60 = (g_object_unref (__temp60), NULL)));
	vala_symbol_add (self->priv->current_symbol, vala_variable_declarator_get_name (decl), vala_code_node_get_symbol (VALA_CODE_NODE (decl)));
	ValaBlock* block = VALA_BLOCK (vala_symbol_get_node (self->priv->current_symbol));
	vala_block_add_local_variable (block, decl);
	vala_symbol_set_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl)), TRUE);
}


static void vala_semantic_analyzer_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_EXPRESSION_STATEMENT (stmt));
	if (vala_expression_get_static_type (vala_expression_statement_get_expression (stmt)) != NULL && vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_expression_statement_get_expression (stmt)))) {
		vala_report_warning (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Short-living reference");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_IF_STATEMENT (stmt));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_if_statement_get_condition (stmt)))) {
		/* if there was an error in the condition, skip this check */
		vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE);
		return;
	}
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_if_statement_get_condition (stmt))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
		vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_if_statement_get_condition (stmt))), "Condition must be boolean");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_WHILE_STATEMENT (stmt));
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_while_statement_get_condition (stmt))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
		vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_while_statement_get_condition (stmt))), "Condition must be boolean");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FOR_STATEMENT (stmt));
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_for_statement_get_condition (stmt))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
		vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_for_statement_get_condition (stmt))), "Condition must be boolean");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_begin_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (stmt));
	if (vala_type_reference_get_data_type (vala_foreach_statement_get_type_reference (stmt)) != NULL) {
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_foreach_statement_get_type_reference (stmt)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	}
	ValaVariableDeclarator* __temp61 = NULL;
	vala_foreach_statement_set_variable_declarator (stmt, (__temp61 = vala_variable_declarator_new (vala_foreach_statement_get_variable_name (stmt), NULL, NULL)));
	(__temp61 == NULL ? NULL : (__temp61 = (g_object_unref (__temp61), NULL)));
	vala_variable_declarator_set_type_reference (vala_foreach_statement_get_variable_declarator (stmt), vala_foreach_statement_get_type_reference (stmt));
	ValaSymbol* __temp62 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt)), (__temp62 = vala_symbol_new (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt)))));
	(__temp62 == NULL ? NULL : (__temp62 = (g_object_unref (__temp62), NULL)));
	vala_symbol_add (self->priv->current_symbol, vala_foreach_statement_get_variable_name (stmt), vala_code_node_get_symbol (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt))));
}


static void vala_semantic_analyzer_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (stmt));
	if (self->priv->current_return_type == NULL) {
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return not allowed in this context");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) == NULL && vala_type_reference_get_data_type (self->priv->current_return_type) != NULL) {
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return without value in non-void function");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && vala_type_reference_get_data_type (self->priv->current_return_type) == NULL && vala_type_reference_get_type_parameter (self->priv->current_return_type) == NULL) {
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return with value in void function");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && !vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt)), self->priv->current_return_type)) {
		char* __temp65 = NULL;
		char* __temp64 = NULL;
		char* __temp63 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), (__temp65 = g_strdup_printf ("Return: Cannot convert from `%s' to `%s'", (__temp63 = vala_type_reference_to_string (vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt)))), (__temp64 = vala_type_reference_to_string (self->priv->current_return_type)))));
		(__temp65 == NULL ? NULL : (__temp65 = (g_free (__temp65), NULL)));
		(__temp64 == NULL ? NULL : (__temp64 = (g_free (__temp64), NULL)));
		(__temp63 == NULL ? NULL : (__temp63 = (g_free (__temp63), NULL)));
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt))) && !vala_type_reference_get_transfers_ownership (self->priv->current_return_type)) {
		vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return value transfers ownership but method return type hasn't been declared to transfer ownership");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt)) != NULL && VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt)))) && vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt))) && !vala_type_reference_get_transfers_ownership (self->priv->current_return_type)) {
		vala_report_warning (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Local variable with strong reference used as return value and method return type hasn't been declared to transfer ownership");
	}
}


static void vala_semantic_analyzer_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BOOLEAN_LITERAL (expr));
	vala_literal_set_static_type (VALA_LITERAL (expr), self->priv->bool_type);
}


static void vala_semantic_analyzer_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CHARACTER_LITERAL (expr));
	ValaTypeReference* __temp66 = NULL;
	vala_literal_set_static_type (VALA_LITERAL (expr), (__temp66 = vala_type_reference_new ()));
	(__temp66 == NULL ? NULL : (__temp66 = (g_object_unref (__temp66), NULL)));
	vala_type_reference_set_data_type (vala_literal_get_static_type (VALA_LITERAL (expr)), VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "char"))));
}


static void vala_semantic_analyzer_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INTEGER_LITERAL (expr));
	ValaTypeReference* __temp67 = NULL;
	vala_literal_set_static_type (VALA_LITERAL (expr), (__temp67 = vala_type_reference_new ()));
	(__temp67 == NULL ? NULL : (__temp67 = (g_object_unref (__temp67), NULL)));
	vala_type_reference_set_data_type (vala_literal_get_static_type (VALA_LITERAL (expr)), VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int"))));
}


static void vala_semantic_analyzer_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_REAL_LITERAL (expr));
	ValaTypeReference* __temp68 = NULL;
	vala_literal_set_static_type (VALA_LITERAL (expr), (__temp68 = vala_type_reference_new ()));
	(__temp68 == NULL ? NULL : (__temp68 = (g_object_unref (__temp68), NULL)));
	vala_type_reference_set_data_type (vala_literal_get_static_type (VALA_LITERAL (expr)), VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "double"))));
}


static void vala_semantic_analyzer_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_STRING_LITERAL (expr));
	ValaTypeReference* __temp69 = NULL;
	vala_literal_set_static_type (VALA_LITERAL (expr), (__temp69 = vala_type_reference_copy (self->priv->string_type)));
	(__temp69 == NULL ? NULL : (__temp69 = (g_object_unref (__temp69), NULL)));
	vala_type_reference_set_non_null (vala_literal_get_static_type (VALA_LITERAL (expr)), TRUE);
}


static void vala_semantic_analyzer_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NULL_LITERAL (expr));
	/* empty TypeReference represents null */
	ValaTypeReference* __temp70 = NULL;
	vala_literal_set_static_type (VALA_LITERAL (expr), (__temp70 = vala_type_reference_new ()));
	(__temp70 == NULL ? NULL : (__temp70 = (g_object_unref (__temp70), NULL)));
}


static void vala_semantic_analyzer_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_LITERAL_EXPRESSION (expr));
	vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_literal_get_static_type (vala_literal_expression_get_literal (expr)));
}


static ValaTypeReference* vala_semantic_analyzer_get_static_type_for_node (ValaSemanticAnalyzer* self, ValaCodeNode* node)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (VALA_IS_CODE_NODE (node), NULL);
	if (VALA_IS_FIELD (node)) {
		ValaField* f = VALA_FIELD (node);
		return g_object_ref (vala_field_get_type_reference (f));
	} else {
		if (VALA_IS_CONSTANT (node)) {
			ValaConstant* c = VALA_CONSTANT (node);
			return g_object_ref (vala_constant_get_type_reference (c));
		} else {
			if (VALA_IS_PROPERTY (node)) {
				ValaProperty* prop = VALA_PROPERTY (node);
				ValaTypeReference* type = vala_type_reference_copy (vala_property_get_type_reference (prop));
				vala_type_reference_set_takes_ownership (type, FALSE);
				ValaTypeReference* __temp73 = NULL;
				return (__temp73 = g_object_ref (type), (type == NULL ? NULL : (type = (g_object_unref (type), NULL))), __temp73);
				(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
			} else {
				if (VALA_IS_FORMAL_PARAMETER (node)) {
					ValaFormalParameter* p = VALA_FORMAL_PARAMETER (node);
					ValaTypeReference* __temp74 = NULL;
					return (__temp74 = vala_formal_parameter_get_type_reference (p), (__temp74 == NULL ? __temp74 : g_object_ref (__temp74)));
				} else {
					if (VALA_IS_TYPE_REFERENCE (node)) {
						ValaTypeReference* __temp76 = NULL;
						return (__temp76 = VALA_TYPE_REFERENCE (node), (__temp76 == NULL ? __temp76 : g_object_ref (__temp76)));
					} else {
						if (VALA_IS_VARIABLE_DECLARATOR (node)) {
							ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (node);
							ValaTypeReference* __temp78 = NULL;
							return (__temp78 = vala_variable_declarator_get_type_reference (decl), (__temp78 == NULL ? __temp78 : g_object_ref (__temp78)));
						} else {
							if (VALA_IS_ENUM_VALUE (node)) {
								ValaTypeReference* type = vala_type_reference_new ();
								vala_type_reference_set_data_type (type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (node)))));
								ValaTypeReference* __temp80 = NULL;
								ValaTypeReference* __temp81 = NULL;
								return (__temp81 = (__temp80 = type, (__temp80 == NULL ? __temp80 : g_object_ref (__temp80))), (type == NULL ? NULL : (type = (g_object_unref (type), NULL))), __temp81);
								(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
							}
						}
					}
				}
			}
		}
	}
	return NULL;
}


static ValaSymbol* vala_semantic_analyzer_symbol_lookup_inherited (ValaSemanticAnalyzer* self, ValaSymbol* sym, char* name)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (VALA_IS_SYMBOL (sym), NULL);
	g_return_val_if_fail (name != NULL, NULL);
	ValaSymbol* result = vala_symbol_lookup (sym, name);
	if (result == NULL && VALA_IS_CLASS (vala_symbol_get_node (sym))) {
		ValaClass* cl = VALA_CLASS (vala_symbol_get_node (sym));
		for (cl = vala_class_get_base_class (cl); cl != NULL && result == NULL; cl = vala_class_get_base_class (cl)) {
			result = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (cl)), name);
		}
	}
	return result;
}


static void vala_semantic_analyzer_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PARENTHESIZED_EXPRESSION (expr));
	vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_parenthesized_expression_get_inner (expr)));
}


static ValaDataType* vala_semantic_analyzer_find_parent_type (ValaSemanticAnalyzer* self, ValaSymbol* sym)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (sym == NULL || VALA_IS_SYMBOL (sym), NULL);
	while (sym != NULL) {
		if (VALA_IS_DATA_TYPE (vala_symbol_get_node (sym))) {
			return VALA_DATA_TYPE (vala_symbol_get_node (sym));
		}
		sym = vala_symbol_get_parent_symbol (sym);
	}
	return NULL;
}


static void vala_semantic_analyzer_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (expr));
	ValaSymbol* base_symbol = NULL;
	if (vala_member_access_get_inner (expr) == NULL) {
		base_symbol = self->priv->current_symbol;
		ValaSymbol* sym = self->priv->current_symbol;
		while (sym != NULL && vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
			vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_semantic_analyzer_symbol_lookup_inherited (self, sym, vala_member_access_get_member_name (expr)));
			sym = vala_symbol_get_parent_symbol (sym);
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
			{
				GList* ns_it;
				for (ns_it = self->priv->current_using_directives; ns_it != NULL; ns_it = ns_it->next) {
					ValaNamespaceReference* ns = ns_it->data;
					{
						ValaSymbol* local_sym = vala_symbol_lookup (vala_namespace_reference_get_namespace_symbol (ns), vala_member_access_get_member_name (expr));
						if (local_sym != NULL) {
							if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) != NULL) {
								vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
								char* __temp88 = NULL;
								char* __temp87 = NULL;
								char* __temp86 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp88 = g_strdup_printf ("`%s' is an ambiguous reference between `%s' and `%s'", vala_member_access_get_member_name (expr), (__temp86 = vala_symbol_get_full_name (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)))), (__temp87 = vala_symbol_get_full_name (local_sym)))));
								(__temp88 == NULL ? NULL : (__temp88 = (g_free (__temp88), NULL)));
								(__temp87 == NULL ? NULL : (__temp87 = (g_free (__temp87), NULL)));
								(__temp86 == NULL ? NULL : (__temp86 = (g_free (__temp86), NULL)));
								return;
							}
							vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), local_sym);
						}
					}
				}
			}
		}
	} else {
		if (vala_code_node_get_error (VALA_CODE_NODE (vala_member_access_get_inner (expr)))) {
			/* if there was an error in the inner expression, skip this check */
			vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
			return;
		}
		if (VALA_IS_MEMBER_ACCESS (vala_member_access_get_inner (expr))) {
			ValaMemberAccess* base = VALA_MEMBER_ACCESS (vala_member_access_get_inner (expr));
			base_symbol = vala_expression_get_symbol_reference (VALA_EXPRESSION (base));
			if (VALA_IS_NAMESPACE (vala_symbol_get_node (base_symbol)) || VALA_IS_DATA_TYPE (vala_symbol_get_node (base_symbol))) {
				vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_symbol_lookup (base_symbol, vala_member_access_get_member_name (expr)));
			}
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && vala_expression_get_static_type (vala_member_access_get_inner (expr)) != NULL) {
			base_symbol = vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (expr)))));
			vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_semantic_analyzer_symbol_lookup_inherited (self, base_symbol, vala_member_access_get_member_name (expr)));
		}
	}
	if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		char* __temp90 = NULL;
		char* __temp89 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp90 = g_strdup_printf ("The name `%s' does not exist in the context of `%s'", vala_member_access_get_member_name (expr), (__temp89 = vala_symbol_get_full_name (base_symbol)))));
		(__temp90 == NULL ? NULL : (__temp90 = (g_free (__temp90), NULL)));
		(__temp89 == NULL ? NULL : (__temp89 = (g_free (__temp89), NULL)));
		return;
	}
	ValaCodeNode* member = vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
	ValaMemberAccessibility access = VALA_MEMBER_ACCESSIBILITY_PUBLIC;
	if (VALA_IS_FIELD (member)) {
		access = (VALA_FIELD (member))->access;
	} else {
		if (VALA_IS_METHOD (member)) {
			access = (VALA_METHOD (member))->access;
		}
	}
	if (access != VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		ValaDataType* target_type = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (member))));
		ValaDataType* this_type = vala_semantic_analyzer_find_parent_type (self, self->priv->current_symbol);
		if (target_type != this_type) {
			vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
			char* __temp92 = NULL;
			char* __temp91 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp92 = g_strdup_printf ("Access to private member `%s' denied", (__temp91 = vala_symbol_get_full_name (vala_code_node_get_symbol (member))))));
			(__temp92 == NULL ? NULL : (__temp92 = (g_free (__temp92), NULL)));
			(__temp91 == NULL ? NULL : (__temp91 = (g_free (__temp91), NULL)));
			return;
		}
	}
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	ValaTypeReference* __temp93 = NULL;
	vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp93 = vala_semantic_analyzer_get_static_type_for_node (self, vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))));
	(__temp93 == NULL ? NULL : (__temp93 = (g_object_unref (__temp93), NULL)));
}


static gboolean vala_semantic_analyzer_is_type_compatible (ValaSemanticAnalyzer* self, ValaTypeReference* expression_type, ValaTypeReference* expected_type)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (expression_type), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (expected_type), FALSE);
	/* only null is compatible to null */
	if (vala_type_reference_get_data_type (expected_type) == NULL && vala_type_reference_get_type_parameter (expected_type) == NULL) {
		return (vala_type_reference_get_data_type (expression_type) == NULL && vala_type_reference_get_type_parameter (expected_type) == NULL);
	}
	/* null can be casted to any reference or array type */
	if (vala_type_reference_get_data_type (expression_type) == NULL && (vala_type_reference_get_type_parameter (expected_type) != NULL || vala_data_type_is_reference_type (vala_type_reference_get_data_type (expected_type)) || vala_type_reference_get_reference_to_value_type (expected_type) || VALA_IS_ARRAY (vala_type_reference_get_data_type (expected_type)) || VALA_IS_CALLBACK (vala_type_reference_get_data_type (expected_type)) || vala_type_reference_get_data_type (expected_type) == self->priv->pointer_type)) {
		return TRUE;
	}
	/* temporarily ignore type parameters */
	if (vala_type_reference_get_type_parameter (expected_type) != NULL) {
		return TRUE;
	}
	if (VALA_IS_ARRAY (vala_type_reference_get_data_type (expression_type)) != VALA_IS_ARRAY (vala_type_reference_get_data_type (expected_type))) {
		return FALSE;
	}
	if (vala_type_reference_get_data_type (expression_type) == vala_type_reference_get_data_type (expected_type)) {
		return TRUE;
	}
	/* int may be implicitly casted to long */
	if (vala_type_reference_get_data_type (expression_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int")) && vala_type_reference_get_data_type (expected_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "long"))) {
		return TRUE;
	}
	/* int may be implicitly casted to ulong */
	if (vala_type_reference_get_data_type (expression_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int")) && vala_type_reference_get_data_type (expected_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "ulong"))) {
		return TRUE;
	}
	/* uint may be implicitly casted to long */
	if (vala_type_reference_get_data_type (expression_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "uint")) && vala_type_reference_get_data_type (expected_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "long"))) {
		return TRUE;
	}
	/* uint may be implicitly casted to ulong */
	if (vala_type_reference_get_data_type (expression_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "uint")) && vala_type_reference_get_data_type (expected_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "ulong"))) {
		return TRUE;
	}
	/* int may be implicitly casted to uint */
	if (vala_type_reference_get_data_type (expression_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int")) && vala_type_reference_get_data_type (expected_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "uint"))) {
		return TRUE;
	}
	/* uint may be implicitly casted to int */
	if (vala_type_reference_get_data_type (expression_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "uint")) && vala_type_reference_get_data_type (expected_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int"))) {
		return TRUE;
	}
	/* long may be implicitly casted to ulong */
	if (vala_type_reference_get_data_type (expression_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "long")) && vala_type_reference_get_data_type (expected_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "ulong"))) {
		return TRUE;
	}
	/* ulong may be implicitly casted to long */
	if (vala_type_reference_get_data_type (expression_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "ulong")) && vala_type_reference_get_data_type (expected_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "long"))) {
		return TRUE;
	}
	/* int may be implicitly casted to double */
	if (vala_type_reference_get_data_type (expression_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int")) && vala_type_reference_get_data_type (expected_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "double"))) {
		return TRUE;
	}
	/* char may be implicitly casted to unichar */
	if (vala_type_reference_get_data_type (expression_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "char")) && vala_type_reference_get_data_type (expected_type) == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "unichar"))) {
		return TRUE;
	}
	return vala_data_type_is_subtype_of (vala_type_reference_get_data_type (expression_type), vala_type_reference_get_data_type (expected_type));
}


static void vala_semantic_analyzer_real_visit_begin_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INVOCATION_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_invocation_expression_get_call (expr)))) {
		/* if method resolving didn't succeed, skip this check */
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		return;
	}
	ValaSymbol* msym = vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr));
	if (msym == NULL) {
		/* if no symbol found, skip this check */
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		return;
	}
	GList* params = NULL;
	if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (msym))) {
		ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (msym));
		if (VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
			ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)));
			GList* __temp110 = NULL;
			params = (__temp110 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp110);
		} else {
			vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "invocation not supported in this context");
			(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
			return;
		}
	} else {
		if (VALA_IS_FORMAL_PARAMETER (vala_symbol_get_node (msym))) {
			ValaFormalParameter* param = VALA_FORMAL_PARAMETER (vala_symbol_get_node (msym));
			if (VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)))) {
				ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)));
				GList* __temp111 = NULL;
				params = (__temp111 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp111);
			} else {
				vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "invocation not supported in this context");
				(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
				return;
			}
		} else {
			if (VALA_IS_FIELD (vala_symbol_get_node (msym))) {
				ValaField* f = VALA_FIELD (vala_symbol_get_node (msym));
				if (VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_field_get_type_reference (f)))) {
					ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_field_get_type_reference (f)));
					GList* __temp112 = NULL;
					params = (__temp112 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp112);
				} else {
					vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "invocation not supported in this context");
					(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
					return;
				}
			} else {
				if (VALA_IS_METHOD (vala_symbol_get_node (msym))) {
					ValaMethod* m = VALA_METHOD (vala_symbol_get_node (msym));
					GList* __temp113 = NULL;
					params = (__temp113 = vala_method_get_parameters (m), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp113);
				} else {
					if (VALA_IS_SIGNAL (vala_symbol_get_node (msym))) {
						ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (msym));
						GList* __temp114 = NULL;
						params = (__temp114 = vala_signal_get_parameters (sig), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp114);
					} else {
						vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "invocation not supported in this context");
						(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
						return;
					}
				}
			}
		}
	}
	GList* args = vala_invocation_expression_get_argument_list (expr);
	GList* arg_it = args;
	{
		GList* param_it;
		for (param_it = params; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				if (vala_formal_parameter_get_ellipsis (param)) {
					break;
				}
				if (arg_it != NULL) {
					ValaExpression* arg = VALA_EXPRESSION (arg_it->data);
					/* store expected type for callback parameters */
					vala_expression_set_expected_type (arg, vala_formal_parameter_get_type_reference (param));
					arg_it = arg_it->next;
				}
			}
		}
	}
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
	(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
}


static gboolean vala_semantic_analyzer_check_arguments (ValaSemanticAnalyzer* self, ValaExpression* expr, ValaSymbol* msym, GList* params, GList* args)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_EXPRESSION (expr), FALSE);
	g_return_val_if_fail (VALA_IS_SYMBOL (msym), FALSE);
	GList* arg_it = args;
	gboolean ellipsis = FALSE;
	int i = 0;
	{
		GList* param_it;
		for (param_it = params; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				if (vala_formal_parameter_get_ellipsis (param)) {
					ellipsis = TRUE;
					break;
				}
				/* header file necessary if we need to cast argument */
				if (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)) != NULL) {
					vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
				}
				if (arg_it == NULL) {
					if (vala_formal_parameter_get_default_expression (param) == NULL) {
						vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
						char* __temp116 = NULL;
						char* __temp115 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp116 = g_strdup_printf ("Too few arguments, method `%s' does not take %d arguments", (__temp115 = vala_symbol_get_full_name (msym)), g_list_length (args))));
						(__temp116 == NULL ? NULL : (__temp116 = (g_free (__temp116), NULL)));
						(__temp115 == NULL ? NULL : (__temp115 = (g_free (__temp115), NULL)));
						return FALSE;
					}
				} else {
					ValaExpression* arg = VALA_EXPRESSION (arg_it->data);
					if (vala_expression_get_static_type (arg) != NULL && !vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (arg), vala_formal_parameter_get_type_reference (param))) {
						/* if there was an error in the argument,
						 * i.e. arg.static_type == null, skip type check */
						vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
						char* __temp120 = NULL;
						char* __temp119 = NULL;
						char* __temp118 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp120 = g_strdup_printf ("Argument %d: Cannot convert from `%s' to `%s'", i + 1, (__temp118 = vala_type_reference_to_string (vala_expression_get_static_type (arg))), (__temp119 = vala_type_reference_to_string (vala_formal_parameter_get_type_reference (param))))));
						(__temp120 == NULL ? NULL : (__temp120 = (g_free (__temp120), NULL)));
						(__temp119 == NULL ? NULL : (__temp119 = (g_free (__temp119), NULL)));
						(__temp118 == NULL ? NULL : (__temp118 = (g_free (__temp118), NULL)));
						return FALSE;
					}
					arg_it = arg_it->next;
					i++;
				}
			}
		}
	}
	if (!ellipsis && arg_it != NULL) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		char* __temp123 = NULL;
		char* __temp122 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp123 = g_strdup_printf ("Too many arguments, method `%s' does not take %d arguments", (__temp122 = vala_symbol_get_full_name (msym)), g_list_length (args))));
		(__temp123 == NULL ? NULL : (__temp123 = (g_free (__temp123), NULL)));
		(__temp122 == NULL ? NULL : (__temp122 = (g_free (__temp122), NULL)));
		return FALSE;
	}
	return TRUE;
}


static void vala_semantic_analyzer_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INVOCATION_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (expr))) {
		return;
	}
	ValaSymbol* msym = vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr));
	ValaTypeReference* ret_type = NULL;
	GList* params = NULL;
	if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (msym))) {
		ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (msym));
		ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)));
		ret_type = vala_callback_get_return_type (cb);
		GList* __temp126 = NULL;
		params = (__temp126 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp126);
	} else {
		if (VALA_IS_FORMAL_PARAMETER (vala_symbol_get_node (msym))) {
			ValaFormalParameter* param = VALA_FORMAL_PARAMETER (vala_symbol_get_node (msym));
			ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)));
			ret_type = vala_callback_get_return_type (cb);
			GList* __temp127 = NULL;
			params = (__temp127 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp127);
		} else {
			if (VALA_IS_FIELD (vala_symbol_get_node (msym))) {
				ValaField* f = VALA_FIELD (vala_symbol_get_node (msym));
				ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_field_get_type_reference (f)));
				ret_type = vala_callback_get_return_type (cb);
				GList* __temp128 = NULL;
				params = (__temp128 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp128);
			} else {
				if (VALA_IS_METHOD (vala_symbol_get_node (msym))) {
					ValaMethod* m = VALA_METHOD (vala_symbol_get_node (msym));
					ret_type = vala_method_get_return_type (m);
					GList* __temp129 = NULL;
					params = (__temp129 = vala_method_get_parameters (m), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp129);
				} else {
					if (VALA_IS_SIGNAL (vala_symbol_get_node (msym))) {
						ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (msym));
						ret_type = vala_signal_get_return_type (sig);
						GList* __temp130 = NULL;
						params = (__temp130 = vala_signal_get_parameters (sig), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp130);
					}
				}
			}
		}
	}
	vala_expression_set_static_type (VALA_EXPRESSION (expr), ret_type);
	GList* __temp131 = NULL;
	vala_semantic_analyzer_check_arguments (self, VALA_EXPRESSION (expr), msym, params, (__temp131 = vala_invocation_expression_get_argument_list (expr)));
	(__temp131 == NULL ? NULL : (__temp131 = (g_list_free (__temp131), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
}


static void vala_semantic_analyzer_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_element_access_get_container (expr))) || vala_code_node_get_error (VALA_CODE_NODE (vala_element_access_get_index (expr)))) {
		/* don't proceed if a child expression failed */
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		return;
	}
	/* assign a static_type when possible */
	if (VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_element_access_get_container (expr))))) {
		ValaTypeReference* __temp132 = NULL;
		vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp132 = vala_type_reference_new ()));
		(__temp132 == NULL ? NULL : (__temp132 = (g_object_unref (__temp132), NULL)));
		vala_type_reference_set_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr)), vala_array_get_element_type ((VALA_ARRAY (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_element_access_get_container (expr)))))));
	} else {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		char* __temp134 = NULL;
		char* __temp133 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp134 = g_strdup_printf ("The expression `%s' does not denote an Array", (__temp133 = vala_type_reference_to_string (vala_expression_get_static_type (vala_element_access_get_container (expr)))))));
		(__temp134 == NULL ? NULL : (__temp134 = (g_free (__temp134), NULL)));
		(__temp133 == NULL ? NULL : (__temp133 = (g_free (__temp133), NULL)));
	}
	/* check if the index is of type integer */
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_element_access_get_index (expr))) != vala_type_reference_get_data_type (self->priv->int_type)) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		char* __temp136 = NULL;
		char* __temp135 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp136 = g_strdup_printf ("The expression `%s' does not denote an `int' which is needed for element access", (__temp135 = vala_type_reference_to_string (vala_expression_get_static_type (vala_element_access_get_index (expr)))))));
		(__temp136 == NULL ? NULL : (__temp136 = (g_free (__temp136), NULL)));
		(__temp135 == NULL ? NULL : (__temp135 = (g_free (__temp135), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_POSTFIX_EXPRESSION (expr));
	vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_postfix_expression_get_inner (expr)));
}


static void vala_semantic_analyzer_real_visit_end_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_OBJECT_CREATION_EXPRESSION (expr));
	ValaDataType* type = NULL;
	if (vala_object_creation_expression_get_type_reference (expr) == NULL) {
		if (vala_object_creation_expression_get_member_name (expr) == NULL) {
			vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Incomplete object creation expression");
			return;
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (vala_object_creation_expression_get_member_name (expr))) == NULL) {
			vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
			return;
		}
		ValaCodeNode* constructor_node = vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (vala_object_creation_expression_get_member_name (expr))));
		ValaCodeNode* type_node = vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (vala_object_creation_expression_get_member_name (expr))));
		if (VALA_IS_METHOD (constructor_node)) {
			type_node = vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (constructor_node)));
			ValaMethod* constructor = VALA_METHOD (constructor_node);
			if (!vala_method_get_construction (constructor)) {
				vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
				char* __temp138 = NULL;
				char* __temp137 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp138 = g_strdup_printf ("`%s' is not a construction method", (__temp137 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (constructor)))))));
				(__temp138 == NULL ? NULL : (__temp138 = (g_free (__temp138), NULL)));
				(__temp137 == NULL ? NULL : (__temp137 = (g_free (__temp137), NULL)));
				return;
			}
			vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (constructor)));
		}
		if (VALA_IS_CLASS (type_node) || VALA_IS_STRUCT (type_node)) {
			type = VALA_DATA_TYPE (type_node);
		} else {
			vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
			char* __temp140 = NULL;
			char* __temp139 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp140 = g_strdup_printf ("`%s' is not a class or struct", (__temp139 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (type)))))));
			(__temp140 == NULL ? NULL : (__temp140 = (g_free (__temp140), NULL)));
			(__temp139 == NULL ? NULL : (__temp139 = (g_free (__temp139), NULL)));
			return;
		}
	}
	if (!vala_data_type_is_reference_type (type)) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		char* __temp142 = NULL;
		char* __temp141 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp142 = g_strdup_printf ("Can't create instance of value type `%s'", (__temp141 = vala_type_reference_to_string (vala_object_creation_expression_get_type_reference (expr))))));
		(__temp142 == NULL ? NULL : (__temp142 = (g_free (__temp142), NULL)));
		(__temp141 == NULL ? NULL : (__temp141 = (g_free (__temp141), NULL)));
		return;
	}
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	ValaTypeReference* __temp143 = NULL;
	vala_object_creation_expression_set_type_reference (expr, (__temp143 = vala_type_reference_new ()));
	(__temp143 == NULL ? NULL : (__temp143 = (g_object_unref (__temp143), NULL)));
	vala_type_reference_set_data_type (vala_object_creation_expression_get_type_reference (expr), type);
	ValaTypeReference* __temp144 = NULL;
	vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp144 = vala_type_reference_copy (vala_object_creation_expression_get_type_reference (expr))));
	(__temp144 == NULL ? NULL : (__temp144 = (g_object_unref (__temp144), NULL)));
	vala_type_reference_set_transfers_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE);
	if (VALA_IS_CLASS (type)) {
		ValaClass* cl = VALA_CLASS (type);
		if (vala_class_get_is_abstract (cl)) {
			vala_expression_set_static_type (VALA_EXPRESSION (expr), NULL);
			vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
			char* __temp146 = NULL;
			char* __temp145 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp146 = g_strdup_printf ("Can't create instance of abstract class `%s'", (__temp145 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cl)))))));
			(__temp146 == NULL ? NULL : (__temp146 = (g_free (__temp146), NULL)));
			(__temp145 == NULL ? NULL : (__temp145 = (g_free (__temp145), NULL)));
			return;
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && vala_class_get_default_construction_method (cl) != NULL) {
			vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (vala_class_get_default_construction_method (cl))));
		}
		while (cl != NULL) {
			if (cl == self->priv->initially_unowned_type) {
				vala_type_reference_set_floating_reference (vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE);
				break;
			}
			cl = vala_class_get_base_class (cl);
		}
	} else {
		if (VALA_IS_STRUCT (type)) {
			ValaStruct* st = VALA_STRUCT (type);
			if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && vala_struct_get_default_construction_method (st) != NULL) {
				vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (vala_struct_get_default_construction_method (st))));
			}
		}
	}
	GList* __temp147 = NULL;
	gboolean __temp148;
	if ((__temp148 = vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && g_list_length ((__temp147 = vala_object_creation_expression_get_argument_list (expr))) != 0, (__temp147 == NULL ? NULL : (__temp147 = (g_list_free (__temp147), NULL))), __temp148)) {
		vala_expression_set_static_type (VALA_EXPRESSION (expr), NULL);
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		char* __temp150 = NULL;
		char* __temp149 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp150 = g_strdup_printf ("No arguments allowed when constructing type `%s'", (__temp149 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (type)))))));
		(__temp150 == NULL ? NULL : (__temp150 = (g_free (__temp150), NULL)));
		(__temp149 == NULL ? NULL : (__temp149 = (g_free (__temp149), NULL)));
		return;
	}
	if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) != NULL) {
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))));
		GList* __temp152 = NULL;
		GList* __temp151 = NULL;
		vala_semantic_analyzer_check_arguments (self, VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp151 = vala_method_get_parameters (m)), (__temp152 = vala_object_creation_expression_get_argument_list (expr)));
		(__temp152 == NULL ? NULL : (__temp152 = (g_list_free (__temp152), NULL)));
		(__temp151 == NULL ? NULL : (__temp151 = (g_list_free (__temp151), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_UNARY_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_unary_expression_get_inner (expr)))) {
		/* if there was an error in the inner expression, skip type check */
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		return;
	}
	if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_PLUS || vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_MINUS) {
		/* integer or floating point type*/
		vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr)));
	} else {
		if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_LOGICAL_NEGATION) {
			/* boolean type*/
			vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr)));
		} else {
			if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_BITWISE_COMPLEMENT) {
				/* integer type*/
				vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr)));
			} else {
				if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_INCREMENT) {
					/* integer type*/
					vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr)));
				} else {
					if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_DECREMENT) {
						/* integer type*/
						vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr)));
					} else {
						if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_REF) {
							/* value type*/
							vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr)));
						} else {
							if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_OUT) {
								/* reference type*/
								vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr)));
							} else {
								vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "internal error: unsupported unary operator");
								return;
							}
						}
					}
				}
			}
		}
	}
}


static void vala_semantic_analyzer_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CAST_EXPRESSION (expr));
	if (vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr)) == NULL) {
		return;
	}
	/* if type resolving didn't succeed, skip this check */
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_cast_expression_get_type_reference (expr));
}


static gboolean vala_semantic_analyzer_check_binary_type (ValaSemanticAnalyzer* self, ValaBinaryExpression* expr, char* operation)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_BINARY_EXPRESSION (expr), FALSE);
	g_return_val_if_fail (operation != NULL, FALSE);
	if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_binary_expression_get_right (expr)), vala_expression_get_static_type (vala_binary_expression_get_left (expr)))) {
		char* __temp155 = NULL;
		char* __temp154 = NULL;
		char* __temp153 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp155 = g_strdup_printf ("%s: Cannot convert from `%s' to `%s'", operation, (__temp153 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_right (expr)))), (__temp154 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))))));
		(__temp155 == NULL ? NULL : (__temp155 = (g_free (__temp155), NULL)));
		(__temp154 == NULL ? NULL : (__temp154 = (g_free (__temp154), NULL)));
		(__temp153 == NULL ? NULL : (__temp153 = (g_free (__temp153), NULL)));
		return FALSE;
	}
	return TRUE;
}


static void vala_semantic_analyzer_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BINARY_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_binary_expression_get_left (expr))) || vala_code_node_get_error (VALA_CODE_NODE (vala_binary_expression_get_right (expr)))) {
		/* if there were any errors in inner expressions, skip type check */
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		return;
	}
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))) == vala_type_reference_get_data_type (self->priv->string_type) && vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_PLUS) {
		if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))) != vala_type_reference_get_data_type (self->priv->string_type)) {
			vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Operands must be strings");
			return;
		}
		vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->string_type);
	} else {
		if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_PLUS || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MINUS || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MUL || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_DIV) {
			/* TODO: check for integer or floating point type in expr.left*/
			if (!vala_semantic_analyzer_check_binary_type (self, expr, "Arithmetic operation")) {
				vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
				return;
			}
			vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_binary_expression_get_left (expr)));
		} else {
			if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MOD || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_LEFT || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_RIGHT || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_XOR) {
				/* TODO: check for integer type in expr.left*/
				if (!vala_semantic_analyzer_check_binary_type (self, expr, "Arithmetic operation")) {
					vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
					return;
				}
				vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_binary_expression_get_left (expr)));
			} else {
				if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN_OR_EQUAL || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL) {
					if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))) == vala_type_reference_get_data_type (self->priv->string_type) && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))) == vala_type_reference_get_data_type (self->priv->string_type)) {
						/* string comparison: convert to a.collate (b) OP 0 */
						ValaInvocationExpression* cmp_call = vala_invocation_expression_new (VALA_EXPRESSION (vala_member_access_new (vala_binary_expression_get_left (expr), "collate", NULL)), NULL);
						vala_invocation_expression_add_argument (cmp_call, vala_binary_expression_get_right (expr));
						vala_binary_expression_set_left (expr, VALA_EXPRESSION (cmp_call));
						ValaLiteralExpression* __temp158 = NULL;
						vala_binary_expression_set_right (expr, VALA_EXPRESSION ((__temp158 = vala_literal_expression_new (VALA_LITERAL (vala_integer_literal_new ("0", NULL)), NULL))));
						(__temp158 == NULL ? NULL : (__temp158 = (g_object_unref (__temp158), NULL)));
						vala_code_node_accept (VALA_CODE_NODE (vala_binary_expression_get_left (expr)), VALA_CODE_VISITOR (self));
						(cmp_call == NULL ? NULL : (cmp_call = (g_object_unref (cmp_call), NULL)));
					} else {
						/* TODO: check for integer or floating point type in expr.left */
						if (!vala_semantic_analyzer_check_binary_type (self, expr, "Relational operation")) {
							vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
							return;
						}
					}
					vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type);
				} else {
					if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_EQUALITY || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_INEQUALITY) {
						/* relational operation */
						if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_binary_expression_get_right (expr)), vala_expression_get_static_type (vala_binary_expression_get_left (expr))) && !vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_binary_expression_get_left (expr)), vala_expression_get_static_type (vala_binary_expression_get_right (expr)))) {
							char* __temp161 = NULL;
							char* __temp160 = NULL;
							char* __temp159 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp161 = g_strdup_printf ("Equality operation: `%s' and `%s' are incompatible, comparison would always evaluate to false", (__temp159 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_right (expr)))), (__temp160 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))))));
							(__temp161 == NULL ? NULL : (__temp161 = (g_free (__temp161), NULL)));
							(__temp160 == NULL ? NULL : (__temp160 = (g_free (__temp160), NULL)));
							(__temp159 == NULL ? NULL : (__temp159 = (g_free (__temp159), NULL)));
							vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
							return;
						}
						if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))) == vala_type_reference_get_data_type (self->priv->string_type) && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))) == vala_type_reference_get_data_type (self->priv->string_type)) {
							/* string comparison: convert to a.collate (b) OP 0 */
							ValaInvocationExpression* cmp_call = vala_invocation_expression_new (VALA_EXPRESSION (vala_member_access_new (vala_binary_expression_get_left (expr), "collate", NULL)), NULL);
							vala_invocation_expression_add_argument (cmp_call, vala_binary_expression_get_right (expr));
							vala_binary_expression_set_left (expr, VALA_EXPRESSION (cmp_call));
							ValaLiteralExpression* __temp162 = NULL;
							vala_binary_expression_set_right (expr, VALA_EXPRESSION ((__temp162 = vala_literal_expression_new (VALA_LITERAL (vala_integer_literal_new ("0", NULL)), NULL))));
							(__temp162 == NULL ? NULL : (__temp162 = (g_object_unref (__temp162), NULL)));
							vala_code_node_accept (VALA_CODE_NODE (vala_binary_expression_get_left (expr)), VALA_CODE_VISITOR (self));
							(cmp_call == NULL ? NULL : (cmp_call = (g_object_unref (cmp_call), NULL)));
						}
						vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type);
					} else {
						if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_AND || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_OR) {
							/* integer type or flags type*/
							vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_binary_expression_get_left (expr)));
						} else {
							if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_AND || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_OR) {
								if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))) != vala_type_reference_get_data_type (self->priv->bool_type) || vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
									vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
									vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Operands must be boolean");
								}
								vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type);
							} else {
								g_assert_not_reached ();
							}
						}
					}
				}
			}
		}
	}
}


static void vala_semantic_analyzer_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_TYPE_CHECK (expr));
	if (vala_type_reference_get_data_type (vala_type_check_get_type_reference (expr)) == NULL) {
		/* if type resolving didn't succeed, skip this check */
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		return;
	}
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_type_check_get_type_reference (expr)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type);
}


static void vala_semantic_analyzer_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONDITIONAL_EXPRESSION (expr));
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_conditional_expression_get_condition (expr))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_conditional_expression_get_condition (expr))), "Condition must be boolean");
		return;
	}
	/* FIXME: use greatest lower bound in the type hierarchy 
	 FIXME: support memory management */
	vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_conditional_expression_get_true_expression (expr)));
}


static char* vala_semantic_analyzer_get_lambda_name (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	char* result = g_strdup_printf ("__lambda%d", self->priv->next_lambda_id);
	self->priv->next_lambda_id++;
	char* __temp163 = NULL;
	char* __temp164 = NULL;
	return (__temp164 = (__temp163 = result, (__temp163 == NULL ? __temp163 : g_strdup (__temp163))), (result == NULL ? NULL : (result = (g_free (result), NULL))), __temp164);
	(result == NULL ? NULL : (result = (g_free (result), NULL)));
}


static ValaMethod* vala_semantic_analyzer_find_current_method (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	ValaSymbol* sym = self->priv->current_symbol;
	while (sym != NULL) {
		if (VALA_IS_METHOD (vala_symbol_get_node (sym))) {
			return VALA_METHOD (vala_symbol_get_node (sym));
		}
		sym = vala_symbol_get_parent_symbol (sym);
	}
	return NULL;
}


static void vala_semantic_analyzer_real_visit_begin_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_LAMBDA_EXPRESSION (l));
	if (vala_expression_get_expected_type (VALA_EXPRESSION (l)) == NULL || !(VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (l)))))) {
		vala_code_node_set_error (VALA_CODE_NODE (l), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (l)), "lambda expression not allowed in this context");
		return;
	}
	ValaMethod* current_method = vala_semantic_analyzer_find_current_method (self);
	ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (l))));
	ValaMethod* __temp167 = NULL;
	vala_lambda_expression_set_method (l, (__temp167 = vala_method_new (vala_semantic_analyzer_get_lambda_name (self), vala_callback_get_return_type (cb), NULL)));
	(__temp167 == NULL ? NULL : (__temp167 = (g_object_unref (__temp167), NULL)));
	vala_method_set_instance (vala_lambda_expression_get_method (l), vala_callback_get_instance (cb) && vala_method_get_instance (current_method));
	ValaSymbol* __temp168 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)), (__temp168 = vala_symbol_new (VALA_CODE_NODE (vala_lambda_expression_get_method (l)))));
	(__temp168 == NULL ? NULL : (__temp168 = (g_object_unref (__temp168), NULL)));
	vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))), self->priv->current_symbol);
	GList* lambda_params = vala_lambda_expression_get_parameters (l);
	GList* lambda_param_it = lambda_params;
	{
		GList* cb_param_it;
		for (cb_param_it = vala_callback_get_parameters (cb); cb_param_it != NULL; cb_param_it = cb_param_it->next) {
			ValaFormalParameter* cb_param = cb_param_it->data;
			{
				if (lambda_param_it == NULL) {
					break;
				}
				/* lambda expressions are allowed to have less parameters */
				char* lambda_param = lambda_param_it->data;
				ValaFormalParameter* param = vala_formal_parameter_new (lambda_param, vala_formal_parameter_get_type_reference (cb_param), NULL);
				ValaSymbol* __temp169 = NULL;
				vala_code_node_set_symbol (VALA_CODE_NODE (param), (__temp169 = vala_symbol_new (VALA_CODE_NODE (param))));
				(__temp169 == NULL ? NULL : (__temp169 = (g_object_unref (__temp169), NULL)));
				vala_symbol_add (vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))), vala_formal_parameter_get_name (param), vala_code_node_get_symbol (VALA_CODE_NODE (param)));
				vala_method_add_parameter (vala_lambda_expression_get_method (l), param);
				lambda_param_it = lambda_param_it->next;
				(param == NULL ? NULL : (param = (g_object_unref (param), NULL)));
			}
		}
	}
	if (lambda_param_it != NULL) {
		/* lambda expressions may not expect more parameters */
		vala_code_node_set_error (VALA_CODE_NODE (l), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (l)), "lambda expression: too many parameters");
		(lambda_params == NULL ? NULL : (lambda_params = (g_list_free (lambda_params), NULL)));
		return;
	}
	if (vala_lambda_expression_get_expression_body (l) != NULL) {
		ValaBlock* block = vala_block_new (NULL);
		ValaSymbol* __temp170 = NULL;
		vala_code_node_set_symbol (VALA_CODE_NODE (block), (__temp170 = vala_symbol_new (VALA_CODE_NODE (block))));
		(__temp170 == NULL ? NULL : (__temp170 = (g_object_unref (__temp170), NULL)));
		vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (block)), vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))));
		if (vala_type_reference_get_data_type (vala_method_get_return_type (vala_lambda_expression_get_method (l))) != NULL) {
			ValaReturnStatement* __temp171 = NULL;
			vala_block_add_statement (block, VALA_STATEMENT ((__temp171 = vala_return_statement_new (vala_lambda_expression_get_expression_body (l), NULL))));
			(__temp171 == NULL ? NULL : (__temp171 = (g_object_unref (__temp171), NULL)));
		} else {
			ValaExpressionStatement* __temp172 = NULL;
			vala_block_add_statement (block, VALA_STATEMENT ((__temp172 = vala_expression_statement_new (vala_lambda_expression_get_expression_body (l), NULL))));
			(__temp172 == NULL ? NULL : (__temp172 = (g_object_unref (__temp172), NULL)));
		}
		vala_method_set_body (vala_lambda_expression_get_method (l), block);
		(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
	} else {
		vala_method_set_body (vala_lambda_expression_get_method (l), vala_lambda_expression_get_statement_body (l));
		vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_body (vala_lambda_expression_get_method (l)))), vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))));
	}
	/* lambda expressions should be usable like MemberAccess of a method */
	vala_expression_set_symbol_reference (VALA_EXPRESSION (l), vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))));
	(lambda_params == NULL ? NULL : (lambda_params = (g_list_free (lambda_params), NULL)));
}


static void vala_semantic_analyzer_real_visit_begin_assignment (ValaCodeVisitor* base, ValaAssignment* a)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ASSIGNMENT (a));
	if (!(VALA_IS_MEMBER_ACCESS (vala_assignment_get_left (a)))) {
		vala_code_node_set_error (VALA_CODE_NODE (a), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "unsupported lvalue in assignment");
		return;
	}
	ValaMemberAccess* ma = VALA_MEMBER_ACCESS (vala_assignment_get_left (a));
	if (vala_code_node_get_error (VALA_CODE_NODE (ma)) || vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)) == NULL) {
		vala_code_node_set_error (VALA_CODE_NODE (a), TRUE);
		return;
	}
	/* if no symbol found, skip this check */
	if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))))) {
		ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
		ValaTypeReference* __temp173 = NULL;
		vala_expression_set_expected_type (vala_assignment_get_right (a), (__temp173 = vala_type_reference_new ()));
		(__temp173 == NULL ? NULL : (__temp173 = (g_object_unref (__temp173), NULL)));
		vala_type_reference_set_data_type (vala_expression_get_expected_type (vala_assignment_get_right (a)), VALA_DATA_TYPE (vala_signal_get_callback (sig)));
	}
}


static void vala_semantic_analyzer_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ASSIGNMENT (a));
	if (vala_code_node_get_error (VALA_CODE_NODE (a)) || vala_code_node_get_error (VALA_CODE_NODE (vala_assignment_get_left (a))) || vala_code_node_get_error (VALA_CODE_NODE (vala_assignment_get_right (a)))) {
		vala_code_node_set_error (VALA_CODE_NODE (a), TRUE);
		return;
	}
	ValaMemberAccess* ma = VALA_MEMBER_ACCESS (vala_assignment_get_left (a));
	if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))))) {
		ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
		if (vala_expression_get_symbol_reference (vala_assignment_get_right (a)) == NULL) {
			vala_code_node_set_error (VALA_CODE_NODE (a), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_assignment_get_right (a))), "unsupported expression for signal handler");
			return;
		}
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_right (a))));
		if (vala_method_get_instance (m) && m->access == VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
			/* TODO: generate wrapper function */
			vala_code_node_set_error (VALA_CODE_NODE (ma), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_assignment_get_right (a))), "public instance methods not yet supported as signal handlers");
			return;
		}
		if (vala_method_get_instance (m)) {
			/* instance signal handlers must have the self
			 * parameter at the end
			 * do not use G_CONNECT_SWAPPED as this would
			 * rearrange the parameters for instance
			 * methods and non-instance methods
			 */
			vala_method_set_instance_last (m, TRUE);
		}
	} else {
		if (VALA_IS_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))))) {
			ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
			if (vala_property_get_set_accessor (prop) == NULL) {
				vala_code_node_set_error (VALA_CODE_NODE (ma), TRUE);
				char* __temp175 = NULL;
				char* __temp174 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (ma)), (__temp175 = g_strdup_printf ("Property `%s' is read-only", (__temp174 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))))));
				(__temp175 == NULL ? NULL : (__temp175 = (g_free (__temp175), NULL)));
				(__temp174 == NULL ? NULL : (__temp174 = (g_free (__temp174), NULL)));
				return;
			}
		} else {
			if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))) && vala_expression_get_static_type (vala_assignment_get_right (a)) == NULL) {
				ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
				ValaMemberAccess* right_ma = VALA_MEMBER_ACCESS (vala_assignment_get_right (a));
				if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma)))) && VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
					ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma))));
					ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)));
					/* check whether method matches callback type */
					if (!vala_callback_matches_method (cb, m)) {
						vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE);
						char* __temp178 = NULL;
						char* __temp177 = NULL;
						char* __temp176 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp178 = g_strdup_printf ("declaration of method `%s' doesn't match declaration of callback `%s'", (__temp176 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp177 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cb)))))));
						(__temp178 == NULL ? NULL : (__temp178 = (g_free (__temp178), NULL)));
						(__temp177 == NULL ? NULL : (__temp177 = (g_free (__temp177), NULL)));
						(__temp176 == NULL ? NULL : (__temp176 = (g_free (__temp176), NULL)));
						return;
					}
					vala_expression_set_static_type (vala_assignment_get_right (a), vala_variable_declarator_get_type_reference (decl));
				} else {
					vala_code_node_set_error (VALA_CODE_NODE (a), TRUE);
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Assignment: Invalid callback assignment attempt");
					return;
				}
			} else {
				if (VALA_IS_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))) && vala_expression_get_static_type (vala_assignment_get_right (a)) == NULL) {
					ValaField* f = VALA_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
					ValaMemberAccess* right_ma = VALA_MEMBER_ACCESS (vala_assignment_get_right (a));
					if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma)))) && VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_field_get_type_reference (f)))) {
						ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma))));
						ValaCallback* cb = VALA_CALLBACK (vala_type_reference_get_data_type (vala_field_get_type_reference (f)));
						/* check whether method matches callback type */
						if (!vala_callback_matches_method (cb, m)) {
							vala_code_node_set_error (VALA_CODE_NODE (f), TRUE);
							char* __temp181 = NULL;
							char* __temp180 = NULL;
							char* __temp179 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp181 = g_strdup_printf ("declaration of method `%s' doesn't match declaration of callback `%s'", (__temp179 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp180 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cb)))))));
							(__temp181 == NULL ? NULL : (__temp181 = (g_free (__temp181), NULL)));
							(__temp180 == NULL ? NULL : (__temp180 = (g_free (__temp180), NULL)));
							(__temp179 == NULL ? NULL : (__temp179 = (g_free (__temp179), NULL)));
							return;
						}
						vala_expression_set_static_type (vala_assignment_get_right (a), vala_field_get_type_reference (f));
					} else {
						vala_code_node_set_error (VALA_CODE_NODE (a), TRUE);
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Assignment: Invalid callback assignment attempt");
						return;
					}
				} else {
					if (vala_expression_get_static_type (vala_assignment_get_left (a)) != NULL && vala_expression_get_static_type (vala_assignment_get_right (a)) != NULL) {
						if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_assignment_get_right (a)), vala_expression_get_static_type (vala_assignment_get_left (a)))) {
							/* if there was an error on either side,
							 * i.e. a.{left|right}.static_type == null, skip type check */
							char* __temp184 = NULL;
							char* __temp183 = NULL;
							char* __temp182 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp184 = g_strdup_printf ("Assignment: Cannot convert from `%s' to `%s'", (__temp182 = vala_type_reference_to_string (vala_expression_get_static_type (vala_assignment_get_right (a)))), (__temp183 = vala_type_reference_to_string (vala_expression_get_static_type (vala_assignment_get_left (a)))))));
							(__temp184 == NULL ? NULL : (__temp184 = (g_free (__temp184), NULL)));
							(__temp183 == NULL ? NULL : (__temp183 = (g_free (__temp183), NULL)));
							(__temp182 == NULL ? NULL : (__temp182 = (g_free (__temp182), NULL)));
							return;
						}
						if (vala_semantic_analyzer_get_memory_management (self)) {
							if (vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_assignment_get_right (a)))) {
								/* rhs transfers ownership of the expression */
								if (!vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
									/* lhs doesn't own the value
									 * promote lhs type if it is a local variable
									 * error if it's not a local variable */
									if (!(VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))))) {
										vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Invalid assignment from owned expression to unowned variable");
									}
									vala_type_reference_set_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a)), TRUE);
								}
							} else {
								if (vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
								}
							}
						}
					}
				}
			}
		}
	}
	/* lhs wants to own the value
	 * rhs doesn't transfer the ownership
	 * code generator needs to add reference
	 * increment calls */
	vala_expression_set_static_type (VALA_EXPRESSION (a), vala_expression_get_static_type (vala_assignment_get_left (a)));
}


gboolean vala_semantic_analyzer_get_memory_management (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	return self->priv->_memory_management;
}


void vala_semantic_analyzer_set_memory_management (ValaSemanticAnalyzer* self, gboolean value)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	self->priv->_memory_management = value;
}


void vala_semantic_analyzer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (object);
	switch (property_id) {
		case VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT:
		g_value_set_boolean (value, vala_semantic_analyzer_get_memory_management (self));
		break;
	}
}


void vala_semantic_analyzer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (object);
	switch (property_id) {
		case VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT:
		vala_semantic_analyzer_set_memory_management (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_semantic_analyzer_class_init (ValaSemanticAnalyzerClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaSemanticAnalyzerPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_semantic_analyzer_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_semantic_analyzer_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_semantic_analyzer_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_source_file = vala_semantic_analyzer_real_visit_begin_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_source_file = vala_semantic_analyzer_real_visit_end_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_namespace = vala_semantic_analyzer_real_visit_begin_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_namespace = vala_semantic_analyzer_real_visit_end_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_class = vala_semantic_analyzer_real_visit_begin_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_class = vala_semantic_analyzer_real_visit_end_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_struct = vala_semantic_analyzer_real_visit_begin_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_struct = vala_semantic_analyzer_real_visit_end_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vala_semantic_analyzer_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_method = vala_semantic_analyzer_real_visit_begin_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_method = vala_semantic_analyzer_real_visit_end_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_formal_parameter = vala_semantic_analyzer_real_visit_formal_parameter;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property = vala_semantic_analyzer_real_visit_end_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_property_accessor = vala_semantic_analyzer_real_visit_begin_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property_accessor = vala_semantic_analyzer_real_visit_end_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_constructor = vala_semantic_analyzer_real_visit_begin_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_constructor = vala_semantic_analyzer_real_visit_end_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_destructor = vala_semantic_analyzer_real_visit_begin_destructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_destructor = vala_semantic_analyzer_real_visit_end_destructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_named_argument = vala_semantic_analyzer_real_visit_named_argument;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_block = vala_semantic_analyzer_real_visit_begin_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_block = vala_semantic_analyzer_real_visit_end_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_variable_declarator = vala_semantic_analyzer_real_visit_variable_declarator;
	VALA_CODE_VISITOR_CLASS (klass)->visit_expression_statement = vala_semantic_analyzer_real_visit_expression_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_if_statement = vala_semantic_analyzer_real_visit_if_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_while_statement = vala_semantic_analyzer_real_visit_while_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_for_statement = vala_semantic_analyzer_real_visit_for_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_foreach_statement = vala_semantic_analyzer_real_visit_begin_foreach_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_return_statement = vala_semantic_analyzer_real_visit_return_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_boolean_literal = vala_semantic_analyzer_real_visit_boolean_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_character_literal = vala_semantic_analyzer_real_visit_character_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_integer_literal = vala_semantic_analyzer_real_visit_integer_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_real_literal = vala_semantic_analyzer_real_visit_real_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_string_literal = vala_semantic_analyzer_real_visit_string_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_null_literal = vala_semantic_analyzer_real_visit_null_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_literal_expression = vala_semantic_analyzer_real_visit_literal_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_parenthesized_expression = vala_semantic_analyzer_real_visit_parenthesized_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_member_access = vala_semantic_analyzer_real_visit_member_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_invocation_expression = vala_semantic_analyzer_real_visit_begin_invocation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_invocation_expression = vala_semantic_analyzer_real_visit_end_invocation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_element_access = vala_semantic_analyzer_real_visit_element_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_postfix_expression = vala_semantic_analyzer_real_visit_postfix_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_object_creation_expression = vala_semantic_analyzer_real_visit_end_object_creation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_unary_expression = vala_semantic_analyzer_real_visit_unary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_cast_expression = vala_semantic_analyzer_real_visit_cast_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_binary_expression = vala_semantic_analyzer_real_visit_binary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_type_check = vala_semantic_analyzer_real_visit_type_check;
	VALA_CODE_VISITOR_CLASS (klass)->visit_conditional_expression = vala_semantic_analyzer_real_visit_conditional_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_lambda_expression = vala_semantic_analyzer_real_visit_begin_lambda_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_assignment = vala_semantic_analyzer_real_visit_begin_assignment;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_assignment = vala_semantic_analyzer_real_visit_end_assignment;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT, g_param_spec_boolean ("memory-management", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_semantic_analyzer_init (ValaSemanticAnalyzer * self)
{
	self->priv = VALA_SEMANTIC_ANALYZER_GET_PRIVATE (self);
	self->priv->next_lambda_id = 0;
}


static void vala_semantic_analyzer_dispose (GObject * obj)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (obj);
	(self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL)));
	(self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL)));
	(self->priv->current_source_file == NULL ? NULL : (self->priv->current_source_file = (g_object_unref (self->priv->current_source_file), NULL)));
	(self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL)));
	(self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL)));
	(self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL)));
	(self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL)));
	(self->priv->int_type == NULL ? NULL : (self->priv->int_type = (g_object_unref (self->priv->int_type), NULL)));
	(self->priv->pointer_type == NULL ? NULL : (self->priv->pointer_type = (g_object_unref (self->priv->pointer_type), NULL)));
	(self->priv->initially_unowned_type == NULL ? NULL : (self->priv->initially_unowned_type = (g_object_unref (self->priv->initially_unowned_type), NULL)));
	ValaSemanticAnalyzerClass * klass;
	GObjectClass * parent_class;
	klass = VALA_SEMANTIC_ANALYZER_CLASS (g_type_class_peek (VALA_TYPE_SEMANTIC_ANALYZER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_semantic_analyzer_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaSemanticAnalyzerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_semantic_analyzer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaSemanticAnalyzer), 0, (GInstanceInitFunc) vala_semantic_analyzer_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaSemanticAnalyzer", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



