/* valasymbolresolver.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "valasymbolresolver.h"
#include <string.h>
#include "valasymbol.h"
#include "valacodenode.h"
#include "valareport.h"
#include "valasourcereference.h"
#include "valadatatype.h"
#include "valatypeparameter.h"

struct _ValaSymbolResolverPrivate {
	ValaSymbol* root_symbol;
	ValaSymbol* current_scope;
	GList* current_using_directives;
	ValaClass* object_class;
};
#define VALA_SYMBOL_RESOLVER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SYMBOL_RESOLVER, ValaSymbolResolverPrivate))
enum  {
	VALA_SYMBOL_RESOLVER_DUMMY_PROPERTY,
};
static void vala_symbol_resolver_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vala_symbol_resolver_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vala_symbol_resolver_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_symbol_resolver_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_symbol_resolver_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_symbol_resolver_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_symbol_resolver_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_symbol_resolver_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_symbol_resolver_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_symbol_resolver_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_symbol_resolver_real_visit_begin_callback (ValaCodeVisitor* base, ValaCallback* cb);
static void vala_symbol_resolver_real_visit_end_callback (ValaCodeVisitor* base, ValaCallback* cb);
static void vala_symbol_resolver_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p);
static void vala_symbol_resolver_real_visit_namespace_reference (ValaCodeVisitor* base, ValaNamespaceReference* ns);
static void vala_symbol_resolver_real_visit_type_reference (ValaCodeVisitor* base, ValaTypeReference* type);
static void vala_symbol_resolver_dispose (GObject * obj);

/**
 * Resolve symbol names in the specified code context.
 *
 * @param context a code context
 */
void vala_symbol_resolver_resolve (ValaSymbolResolver* self, ValaCodeContext* context)
{
	g_return_if_fail (VALA_IS_SYMBOL_RESOLVER (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	ValaSymbol* __temp0 = NULL;
	self->priv->root_symbol = (__temp0 = g_object_ref (vala_code_context_get_root (context)), (self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL))), __temp0);
	ValaSymbol* __temp2 = NULL;
	ValaSymbol* __temp1 = NULL;
	self->priv->current_scope = (__temp2 = (__temp1 = self->priv->root_symbol, (__temp1 == NULL ? __temp1 : g_object_ref (__temp1))), (self->priv->current_scope == NULL ? NULL : (self->priv->current_scope = (g_object_unref (self->priv->current_scope), NULL))), __temp2);
	ValaClass* __temp4 = NULL;
	ValaClass* __temp3 = NULL;
	self->priv->object_class = (__temp4 = (__temp3 = VALA_CLASS (vala_symbol_get_node (vala_symbol_lookup (vala_symbol_lookup (self->priv->root_symbol, "GLib"), "Object"))), (__temp3 == NULL ? __temp3 : g_object_ref (__temp3))), (self->priv->object_class == NULL ? NULL : (self->priv->object_class = (g_object_unref (self->priv->object_class), NULL))), __temp4);
	vala_code_context_accept (context, VALA_CODE_VISITOR (self));
}


static void vala_symbol_resolver_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* file)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (file));
	GList* __temp5 = NULL;
	self->priv->current_using_directives = (__temp5 = vala_source_file_get_using_directives (file), (self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL))), __temp5);
}


static void vala_symbol_resolver_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* file)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (file));
	GList* __temp6 = NULL;
	self->priv->current_using_directives = (__temp6 = NULL, (self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL))), __temp6);
}


static void vala_symbol_resolver_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	ValaSymbol* __temp8 = NULL;
	ValaSymbol* __temp7 = NULL;
	self->priv->current_scope = (__temp8 = (__temp7 = vala_code_node_get_symbol (VALA_CODE_NODE (ns)), (__temp7 == NULL ? __temp7 : g_object_ref (__temp7))), (self->priv->current_scope == NULL ? NULL : (self->priv->current_scope = (g_object_unref (self->priv->current_scope), NULL))), __temp8);
}


static void vala_symbol_resolver_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	/* don't use current_scope.parent_symbol as that would be null
	 if the current namespace is SourceFile.global_namespace*/
	ValaSymbol* __temp10 = NULL;
	ValaSymbol* __temp9 = NULL;
	self->priv->current_scope = (__temp10 = (__temp9 = self->priv->root_symbol, (__temp9 == NULL ? __temp9 : g_object_ref (__temp9))), (self->priv->current_scope == NULL ? NULL : (self->priv->current_scope = (g_object_unref (self->priv->current_scope), NULL))), __temp10);
}


static void vala_symbol_resolver_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* __temp12 = NULL;
	ValaSymbol* __temp11 = NULL;
	self->priv->current_scope = (__temp12 = (__temp11 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp11 == NULL ? __temp11 : g_object_ref (__temp11))), (self->priv->current_scope == NULL ? NULL : (self->priv->current_scope = (g_object_unref (self->priv->current_scope), NULL))), __temp12);
}


static void vala_symbol_resolver_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	{
		GList* type_it;
		for (type_it = vala_class_get_base_types (cl); type_it != NULL; type_it = type_it->next) {
			ValaTypeReference* type = type_it->data;
			{
				if (VALA_IS_CLASS (vala_type_reference_get_data_type (type))) {
					if (vala_class_get_base_class (cl) != NULL) {
						char* __temp16 = NULL;
						char* __temp15 = NULL;
						char* __temp14 = NULL;
						char* __temp13 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (type)), (__temp16 = g_strdup_printf ("%s: Classes cannot have multiple base classes (`%s' and `%s')", (__temp13 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cl)))), (__temp14 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (vala_class_get_base_class (cl))))), (__temp15 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (type))))))));
						(__temp16 == NULL ? NULL : (__temp16 = (g_free (__temp16), NULL)));
						(__temp15 == NULL ? NULL : (__temp15 = (g_free (__temp15), NULL)));
						(__temp14 == NULL ? NULL : (__temp14 = (g_free (__temp14), NULL)));
						(__temp13 == NULL ? NULL : (__temp13 = (g_free (__temp13), NULL)));
						return;
					}
					vala_class_set_base_class (cl, VALA_CLASS (vala_type_reference_get_data_type (type)));
				}
			}
		}
	}
	if (vala_class_get_base_class (cl) == NULL && cl != self->priv->object_class) {
		vala_class_set_base_class (cl, self->priv->object_class);
	}
	ValaSymbol* __temp18 = NULL;
	ValaSymbol* __temp17 = NULL;
	self->priv->current_scope = (__temp18 = (__temp17 = vala_symbol_get_parent_symbol (self->priv->current_scope), (__temp17 == NULL ? __temp17 : g_object_ref (__temp17))), (self->priv->current_scope == NULL ? NULL : (self->priv->current_scope = (g_object_unref (self->priv->current_scope), NULL))), __temp18);
}


static void vala_symbol_resolver_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaSymbol* __temp20 = NULL;
	ValaSymbol* __temp19 = NULL;
	self->priv->current_scope = (__temp20 = (__temp19 = vala_code_node_get_symbol (VALA_CODE_NODE (st)), (__temp19 == NULL ? __temp19 : g_object_ref (__temp19))), (self->priv->current_scope == NULL ? NULL : (self->priv->current_scope = (g_object_unref (self->priv->current_scope), NULL))), __temp20);
}


static void vala_symbol_resolver_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaSymbol* __temp22 = NULL;
	ValaSymbol* __temp21 = NULL;
	self->priv->current_scope = (__temp22 = (__temp21 = vala_symbol_get_parent_symbol (self->priv->current_scope), (__temp21 == NULL ? __temp21 : g_object_ref (__temp21))), (self->priv->current_scope == NULL ? NULL : (self->priv->current_scope = (g_object_unref (self->priv->current_scope), NULL))), __temp22);
}


static void vala_symbol_resolver_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	ValaSymbol* __temp24 = NULL;
	ValaSymbol* __temp23 = NULL;
	self->priv->current_scope = (__temp24 = (__temp23 = vala_code_node_get_symbol (VALA_CODE_NODE (iface)), (__temp23 == NULL ? __temp23 : g_object_ref (__temp23))), (self->priv->current_scope == NULL ? NULL : (self->priv->current_scope = (g_object_unref (self->priv->current_scope), NULL))), __temp24);
}


static void vala_symbol_resolver_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	ValaSymbol* __temp26 = NULL;
	ValaSymbol* __temp25 = NULL;
	self->priv->current_scope = (__temp26 = (__temp25 = vala_symbol_get_parent_symbol (self->priv->current_scope), (__temp25 == NULL ? __temp25 : g_object_ref (__temp25))), (self->priv->current_scope == NULL ? NULL : (self->priv->current_scope = (g_object_unref (self->priv->current_scope), NULL))), __temp26);
}


static void vala_symbol_resolver_real_visit_begin_callback (ValaCodeVisitor* base, ValaCallback* cb)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_CALLBACK (cb));
	ValaSymbol* __temp28 = NULL;
	ValaSymbol* __temp27 = NULL;
	self->priv->current_scope = (__temp28 = (__temp27 = vala_code_node_get_symbol (VALA_CODE_NODE (cb)), (__temp27 == NULL ? __temp27 : g_object_ref (__temp27))), (self->priv->current_scope == NULL ? NULL : (self->priv->current_scope = (g_object_unref (self->priv->current_scope), NULL))), __temp28);
}


static void vala_symbol_resolver_real_visit_end_callback (ValaCodeVisitor* base, ValaCallback* cb)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_CALLBACK (cb));
	ValaSymbol* __temp30 = NULL;
	ValaSymbol* __temp29 = NULL;
	self->priv->current_scope = (__temp30 = (__temp29 = vala_symbol_get_parent_symbol (self->priv->current_scope), (__temp29 == NULL ? __temp29 : g_object_ref (__temp29))), (self->priv->current_scope == NULL ? NULL : (self->priv->current_scope = (g_object_unref (self->priv->current_scope), NULL))), __temp30);
}


static void vala_symbol_resolver_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_FORMAL_PARAMETER (p));
	if (!vala_formal_parameter_get_ellipsis (p) && vala_type_reference_get_is_ref (vala_formal_parameter_get_type_reference (p))) {
		if ((vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)) != NULL && vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)))) || vala_type_reference_get_type_parameter (vala_formal_parameter_get_type_reference (p)) != NULL) {
			vala_type_reference_set_takes_ownership (vala_formal_parameter_get_type_reference (p), TRUE);
		} else {
			vala_type_reference_set_reference_to_value_type (vala_formal_parameter_get_type_reference (p), TRUE);
		}
	}
}


static void vala_symbol_resolver_real_visit_namespace_reference (ValaCodeVisitor* base, ValaNamespaceReference* ns)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_NAMESPACE_REFERENCE (ns));
	vala_namespace_reference_set_namespace_symbol (ns, vala_symbol_lookup (self->priv->current_scope, vala_namespace_reference_get_name (ns)));
	if (vala_namespace_reference_get_namespace_symbol (ns) == NULL) {
		char* __temp31 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (ns)), (__temp31 = g_strdup_printf ("The namespace name `%s' could not be found", vala_namespace_reference_get_name (ns))));
		(__temp31 == NULL ? NULL : (__temp31 = (g_free (__temp31), NULL)));
		return;
	}
}


static void vala_symbol_resolver_real_visit_type_reference (ValaCodeVisitor* base, ValaTypeReference* type)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (base);
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (type));
	if (g_utf8_collate (vala_type_reference_get_type_name (type), "void") == 0) {
		return;
	}
	if (vala_type_reference_get_namespace_name (type) == NULL) {
		ValaSymbol* sym = NULL;
		ValaSymbol* scope = self->priv->current_scope;
		while (sym == NULL && scope != NULL) {
			sym = vala_symbol_lookup (scope, vala_type_reference_get_type_name (type));
			scope = vala_symbol_get_parent_symbol (scope);
		}
		if (sym == NULL) {
			{
				GList* ns_it;
				for (ns_it = self->priv->current_using_directives; ns_it != NULL; ns_it = ns_it->next) {
					ValaNamespaceReference* ns = ns_it->data;
					{
						ValaSymbol* local_sym = vala_symbol_lookup (vala_namespace_reference_get_namespace_symbol (ns), vala_type_reference_get_type_name (type));
						if (local_sym != NULL) {
							if (sym != NULL) {
								char* __temp34 = NULL;
								char* __temp33 = NULL;
								char* __temp32 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (type)), (__temp34 = g_strdup_printf ("`%s' is an ambiguous reference between `%s' and `%s'", vala_type_reference_get_type_name (type), (__temp32 = vala_symbol_get_full_name (sym)), (__temp33 = vala_symbol_get_full_name (local_sym)))));
								(__temp34 == NULL ? NULL : (__temp34 = (g_free (__temp34), NULL)));
								(__temp33 == NULL ? NULL : (__temp33 = (g_free (__temp33), NULL)));
								(__temp32 == NULL ? NULL : (__temp32 = (g_free (__temp32), NULL)));
								return;
							}
							sym = local_sym;
						}
					}
				}
			}
		}
		if (sym == NULL) {
			char* __temp35 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (type)), (__temp35 = g_strdup_printf ("The type name `%s' could not be found", vala_type_reference_get_type_name (type))));
			(__temp35 == NULL ? NULL : (__temp35 = (g_free (__temp35), NULL)));
			return;
		}
		if (VALA_IS_TYPE_PARAMETER (vala_symbol_get_node (sym))) {
			vala_type_reference_set_type_parameter (type, VALA_TYPE_PARAMETER (vala_symbol_get_node (sym)));
		} else {
			vala_type_reference_set_data_type (type, VALA_DATA_TYPE (vala_symbol_get_node (sym)));
		}
	} else {
		ValaSymbol* ns_symbol = vala_symbol_lookup (self->priv->root_symbol, vala_type_reference_get_namespace_name (type));
		if (ns_symbol == NULL) {
			char* __temp36 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (type)), (__temp36 = g_strdup_printf ("The namespace name `%s' could not be found", vala_type_reference_get_namespace_name (type))));
			(__temp36 == NULL ? NULL : (__temp36 = (g_free (__temp36), NULL)));
			return;
		}
		ValaSymbol* sym = vala_symbol_lookup (ns_symbol, vala_type_reference_get_type_name (type));
		if (sym == NULL) {
			char* __temp37 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (type)), (__temp37 = g_strdup_printf ("The type name `%s' does not exist in the namespace `%s'", vala_type_reference_get_type_name (type), vala_type_reference_get_namespace_name (type))));
			(__temp37 == NULL ? NULL : (__temp37 = (g_free (__temp37), NULL)));
			return;
		}
		vala_type_reference_set_data_type (type, VALA_DATA_TYPE (vala_symbol_get_node (sym)));
	}
	if (vala_type_reference_get_data_type (type) != NULL && !vala_data_type_is_reference_type (vala_type_reference_get_data_type (type))) {
		/* reset takes_ownership for contexts where types
		 * are ref by default (field declarations)
		 */
		vala_type_reference_set_takes_ownership (type, FALSE);
	}
	/* check for array */
	if (vala_type_reference_get_array (type)) {
		vala_type_reference_set_data_type (type, VALA_DATA_TYPE (vala_data_type_get_array (vala_type_reference_get_data_type (type))));
	}
}


void vala_symbol_resolver_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (object);
	switch (property_id) {
	}
}


void vala_symbol_resolver_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (object);
	switch (property_id) {
	}
}


static void vala_symbol_resolver_class_init (ValaSymbolResolverClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaSymbolResolverPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_symbol_resolver_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_symbol_resolver_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_symbol_resolver_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_source_file = vala_symbol_resolver_real_visit_begin_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_source_file = vala_symbol_resolver_real_visit_end_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_namespace = vala_symbol_resolver_real_visit_begin_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_namespace = vala_symbol_resolver_real_visit_end_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_class = vala_symbol_resolver_real_visit_begin_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_class = vala_symbol_resolver_real_visit_end_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_struct = vala_symbol_resolver_real_visit_begin_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_struct = vala_symbol_resolver_real_visit_end_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_interface = vala_symbol_resolver_real_visit_begin_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_interface = vala_symbol_resolver_real_visit_end_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_callback = vala_symbol_resolver_real_visit_begin_callback;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_callback = vala_symbol_resolver_real_visit_end_callback;
	VALA_CODE_VISITOR_CLASS (klass)->visit_formal_parameter = vala_symbol_resolver_real_visit_formal_parameter;
	VALA_CODE_VISITOR_CLASS (klass)->visit_namespace_reference = vala_symbol_resolver_real_visit_namespace_reference;
	VALA_CODE_VISITOR_CLASS (klass)->visit_type_reference = vala_symbol_resolver_real_visit_type_reference;
}


static void vala_symbol_resolver_init (ValaSymbolResolver * self)
{
	self->priv = VALA_SYMBOL_RESOLVER_GET_PRIVATE (self);
}


static void vala_symbol_resolver_dispose (GObject * obj)
{
	ValaSymbolResolver * self = VALA_SYMBOL_RESOLVER (obj);
	(self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL)));
	(self->priv->current_scope == NULL ? NULL : (self->priv->current_scope = (g_object_unref (self->priv->current_scope), NULL)));
	(self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL)));
	(self->priv->object_class == NULL ? NULL : (self->priv->object_class = (g_object_unref (self->priv->object_class), NULL)));
	ValaSymbolResolverClass * klass;
	GObjectClass * parent_class;
	klass = VALA_SYMBOL_RESOLVER_CLASS (g_type_class_peek (VALA_TYPE_SYMBOL_RESOLVER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_symbol_resolver_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaSymbolResolverClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_symbol_resolver_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaSymbolResolver), 0, (GInstanceInitFunc) vala_symbol_resolver_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaSymbolResolver", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



